@@ -1,5 +1,5 @@
-#ifndef SHA_PADLOCK_H
-#define SHA_PADLOCK_H
+#ifndef GNUTLS_LIB_ACCELERATED_X86_SHA_PADLOCK_H
+#define GNUTLS_LIB_ACCELERATED_X86_SHA_PADLOCK_H
 
 #include <nettle/sha.h>
 
@@ -18,11 +18,11 @@ int wrap_padlock_hash_fast(gnutls_digest_algorithm_t algo,
 			   void *digest);
 
 void padlock_sha1_update(struct sha1_ctx *ctx,
-			 unsigned length, const uint8_t * data);
+			 size_t length, const uint8_t * data);
 void padlock_sha256_update(struct sha256_ctx *ctx,
-			   unsigned length, const uint8_t * data);
+			   size_t length, const uint8_t * data);
 void padlock_sha512_update(struct sha512_ctx *ctx,
-			   unsigned length, const uint8_t * data);
+			   size_t length, const uint8_t * data);
 
 extern const struct nettle_hash padlock_sha1;
 extern const struct nettle_hash padlock_sha224;
@@ -30,7 +30,7 @@ extern const struct nettle_hash padlock_sha256;
 extern const struct nettle_hash padlock_sha384;
 extern const struct nettle_hash padlock_sha512;
 
-extern const gnutls_crypto_mac_st _gnutls_hmac_sha_padlock_nano;
-extern const gnutls_crypto_digest_st _gnutls_sha_padlock_nano;
+extern const gnutls_crypto_mac_st _gnutls_hmac_sha_padlock;
+extern const gnutls_crypto_digest_st _gnutls_sha_padlock;
 
-#endif
+#endif /* GNUTLS_LIB_ACCELERATED_X86_SHA_PADLOCK_H */
