@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -84,6 +84,8 @@ static const gnutls_alert_entry sup_alerts[] = {
 	ALERT_ENTRY(GNUTLS_A_NO_APPLICATION_PROTOCOL,
 		    N_
 		    ("No supported application protocol could be negotiated")),
+	ALERT_ENTRY(GNUTLS_A_CERTIFICATE_REQUIRED,
+		    N_("Certificate is required")),
 	{0, NULL, NULL}
 };
 
@@ -163,6 +165,22 @@ gnutls_alert_send(gnutls_session_t session, gnutls_alert_level_t level,
 	_gnutls_record_log("REC: Sending Alert[%d|%d] - %s\n", data[0],
 			   data[1], name);
 
+	if (session->internals.alert_read_func) {
+		record_parameters_st *params;
+
+		ret = _gnutls_epoch_get(session, EPOCH_WRITE_CURRENT, &params);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+		ret = session->internals.alert_read_func(session,
+						     params->write.level,
+						     level,
+						     desc);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+
+		return ret;
+	}
+
 	if ((ret =
 	     _gnutls_send_int(session, GNUTLS_ALERT, -1,
 			      EPOCH_WRITE_CURRENT, data, 2,
@@ -224,6 +242,8 @@ int gnutls_error_to_alert(int err, int *level)
 	case GNUTLS_E_ILLEGAL_SRP_USERNAME:
 	case GNUTLS_E_PK_INVALID_PUBKEY:
 	case GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM:
+	case GNUTLS_E_RECEIVED_DISALLOWED_NAME:
+	case GNUTLS_E_INCOMPATIBLE_SIG_WITH_KEY:
 		ret = GNUTLS_A_ILLEGAL_PARAMETER;
 		_level = GNUTLS_AL_FATAL;
 		break;
@@ -325,6 +345,10 @@ int gnutls_error_to_alert(int err, int *level)
 		ret = GNUTLS_A_UNRECOGNIZED_NAME;
 		_level = GNUTLS_AL_FATAL;
 		break;
+	case GNUTLS_E_CERTIFICATE_REQUIRED:
+		ret = GNUTLS_A_CERTIFICATE_REQUIRED;
+		_level = GNUTLS_AL_FATAL;
+		break;
 	default:
 		ret = GNUTLS_A_INTERNAL_ERROR;
 		_level = GNUTLS_AL_FATAL;
