@@ -55,11 +55,25 @@ selected_certs_set(gnutls_session_t session,
 		   gnutls_status_request_ocsp_func ocsp_func,
 		   void *ocsp_func_ptr);
 
-#define MAX_CLIENT_SIGN_ALGOS 3
+#define MAX_CLIENT_SIGN_ALGOS 5
 #define CERTTYPE_SIZE (MAX_CLIENT_SIGN_ALGOS+1)
-typedef enum CertificateSigType { RSA_SIGN = 1, DSA_SIGN = 2, ECDSA_SIGN = 64
+typedef enum CertificateSigType { RSA_SIGN = 1, DSA_SIGN = 2, ECDSA_SIGN = 64,
+#ifdef ENABLE_GOST
+	GOSTR34102012_256_SIGN = 67,
+	GOSTR34102012_512_SIGN = 68
+#endif
 } CertificateSigType;
 
+enum CertificateSigTypeFlags {
+	RSA_SIGN_FLAG = 1,
+	DSA_SIGN_FLAG = 1 << 1,
+	ECDSA_SIGN_FLAG = 1 << 2,
+#ifdef ENABLE_GOST
+	GOSTR34102012_256_SIGN_FLAG = 1 << 3,
+	GOSTR34102012_512_SIGN_FLAG = 1 << 4
+#endif
+};
+
 /* Moves data from an internal certificate struct (gnutls_pcert_st) to
  * another internal certificate struct (cert_auth_info_t), and deinitializes
  * the former.
@@ -123,7 +137,7 @@ check_pk_algo_in_list(const gnutls_pk_algorithm_t *
  */
 static int cert_get_issuer_dn(gnutls_pcert_st * cert, gnutls_datum_t * odn)
 {
-	ASN1_TYPE dn;
+	asn1_node dn;
 	int len, result;
 	int start, end;
 
@@ -1017,6 +1031,12 @@ inline static int _gnutls_check_supported_sign_algo(CertificateSigType algo)
 		return GNUTLS_PK_DSA;
 	case ECDSA_SIGN:
 		return GNUTLS_PK_EC;
+#ifdef ENABLE_GOST
+	case GOSTR34102012_256_SIGN:
+		return GNUTLS_PK_GOST_12_256;
+	case GOSTR34102012_512_SIGN:
+		return GNUTLS_PK_GOST_12_512;
+#endif
 	}
 
 	return -1;
@@ -1267,10 +1287,11 @@ _gnutls_gen_cert_server_cert_req(gnutls_session_t session,
 				 gnutls_buffer_st * data)
 {
 	gnutls_certificate_credentials_t cred;
-	int ret;
+	int ret, i;
 	uint8_t tmp_data[CERTTYPE_SIZE];
 	const version_entry_st *ver = get_version(session);
 	unsigned init_pos = data->length;
+	enum CertificateSigTypeFlags flags;
 
 	if (unlikely(ver == NULL))
 		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
@@ -1287,12 +1308,74 @@ _gnutls_gen_cert_server_cert_req(gnutls_session_t session,
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
-	tmp_data[0] = CERTTYPE_SIZE - 1;
-	tmp_data[1] = RSA_SIGN;
-	tmp_data[2] = DSA_SIGN;
-	tmp_data[3] = ECDSA_SIGN;	/* only these for now */
+	if (_gnutls_version_has_selectable_sighash(ver)) {
+		size_t j;
 
-	ret = _gnutls_buffer_append_data(data, tmp_data, CERTTYPE_SIZE);
+		flags = 0;
+		for (j = 0; j < session->internals.priorities->sigalg.size; j++) {
+			const gnutls_sign_entry_st *se =
+				session->internals.priorities->sigalg.entry[j];
+			switch (se->pk) {
+			case GNUTLS_PK_RSA:
+			case GNUTLS_PK_RSA_PSS:
+				flags |= RSA_SIGN_FLAG;
+				break;
+			case GNUTLS_PK_DSA:
+				flags |= DSA_SIGN_FLAG;
+				break;
+			case GNUTLS_PK_ECDSA:
+				flags |= ECDSA_SIGN_FLAG;
+				break;
+#ifdef ENABLE_GOST
+			case GNUTLS_PK_GOST_12_256:
+				flags |= GOSTR34102012_256_SIGN_FLAG;
+				break;
+			case GNUTLS_PK_GOST_12_512:
+				flags |= GOSTR34102012_512_SIGN_FLAG;
+				break;
+#endif
+			default:
+				gnutls_assert();
+				_gnutls_debug_log(
+					"%s is unsupported for cert request\n",
+					gnutls_pk_get_name(se->pk));
+			}
+		}
+
+	} else {
+#ifdef ENABLE_GOST
+		if (_gnutls_kx_is_vko_gost(session->security_parameters.
+					   cs->kx_algorithm)) {
+			flags = GOSTR34102012_256_SIGN_FLAG |
+				GOSTR34102012_512_SIGN_FLAG;
+		} else
+#endif
+		{
+			flags = RSA_SIGN_FLAG | DSA_SIGN_FLAG | ECDSA_SIGN_FLAG;
+		}
+	}
+
+	i = 1;
+	if (flags & RSA_SIGN_FLAG) {
+		tmp_data[i++] = RSA_SIGN;
+	}
+	if (flags & DSA_SIGN_FLAG) {
+		tmp_data[i++] = DSA_SIGN;
+	}
+	if (flags & ECDSA_SIGN_FLAG) {
+		tmp_data[i++] = ECDSA_SIGN;
+	}
+#ifdef ENABLE_GOST
+	if (flags & GOSTR34102012_256_SIGN_FLAG) {
+		tmp_data[i++] = GOSTR34102012_256_SIGN;
+	}
+	if (flags & GOSTR34102012_512_SIGN_FLAG) {
+		tmp_data[i++] = GOSTR34102012_512_SIGN;
+	}
+#endif
+	tmp_data[0] = i - 1;
+
+	ret = _gnutls_buffer_append_data(data, tmp_data, i);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -1497,7 +1580,7 @@ int cert_select_sign_algorithm(gnutls_session_t session,
 		return 0;
 	}
 
-	algo = _gnutls_session_get_sign_algo(session, cert, pkey, 0);
+	algo = _gnutls_session_get_sign_algo(session, cert, pkey, 0, cs->kx_algorithm);
 	if (algo == GNUTLS_SIGN_UNKNOWN)
 		return gnutls_assert_val(GNUTLS_E_INCOMPATIBLE_SIG_WITH_KEY);
 
