@@ -43,6 +43,7 @@
 #include "x509/common.h"
 #include "dh.h"
 #include "cert-cred.h"
+#include "intprops.h"
 
 
 /*
@@ -55,15 +56,19 @@ _gnutls_certificate_credential_append_keypair(gnutls_certificate_credentials_t r
 				       gnutls_pcert_st * crt,
 				       int nr)
 {
-	res->sorted_cert_idx = gnutls_realloc_fast(res->sorted_cert_idx,
-						(1 + res->ncerts) *
-						sizeof(unsigned int));
+	if (unlikely(INT_ADD_OVERFLOW(res->ncerts, 1))) {
+		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+	}
+
+	res->sorted_cert_idx = _gnutls_reallocarray_fast(res->sorted_cert_idx,
+							 res->ncerts + 1,
+							 sizeof(unsigned int));
 	if (res->sorted_cert_idx == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
-	res->certs = gnutls_realloc_fast(res->certs,
-					 (1 + res->ncerts) *
-					 sizeof(certs_st));
+	res->certs = _gnutls_reallocarray_fast(res->certs,
+					       res->ncerts + 1,
+					       sizeof(certs_st));
 	if (res->certs == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
@@ -204,7 +209,8 @@ gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 		gnutls_privkey_set_pin_function(key, res->pin.cb,
 						res->pin.data);
 
-	new_pcert_list = gnutls_malloc(sizeof(gnutls_pcert_st) * pcert_list_size);
+	new_pcert_list = _gnutls_reallocarray(NULL, pcert_list_size,
+					      sizeof(gnutls_pcert_st));
 	if (new_pcert_list == NULL) {
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
@@ -451,7 +457,8 @@ static gnutls_pcert_st *alloc_and_load_x509_certs(gnutls_x509_crt_t *
 	if (certs == NULL)
 		return NULL;
 
-	local_certs = gnutls_malloc(sizeof(gnutls_pcert_st) * ncerts);
+	local_certs = _gnutls_reallocarray(NULL, ncerts,
+					   sizeof(gnutls_pcert_st));
 	if (local_certs == NULL) {
 		gnutls_assert();
 		return NULL;
@@ -809,7 +816,7 @@ void gnutls_certificate_set_retrieve_function2
  * gnutls_session_t,
  * const struct gnutls_cert_retr_st *info,
  * gnutls_pcert_st **certs,
- * unsigned int *pcert_length,
+ * unsigned int *certs_length,
  * gnutls_ocsp_data_st **ocsp,
  * unsigned int *ocsp_length,
  * gnutls_privkey_t *privkey,
@@ -823,13 +830,13 @@ void gnutls_certificate_set_retrieve_function2
  * @pk_algos contains a list with server's acceptable public key algorithms.
  * The certificate returned should support the server's given algorithms.
  *
- * The callback should fill-in the following values.
+ * The callback should fill-in the following values:
  *
- * @pcert should contain an allocated list of certificates and public keys.
- * @pcert_length is the size of the previous list.
+ * @certs should contain an allocated list of certificates and public keys.
+ * @certs_length is the size of the previous list.
  * @ocsp should contain an allocated list of OCSP responses.
  * @ocsp_length is the size of the previous list.
- * @pkey is the private key.
+ * @privkey is the private key.
  *
  * If flags in the callback are set to %GNUTLS_CERT_RETR_DEINIT_ALL then
  * all provided values must be allocated using gnutls_malloc(), and will
@@ -837,11 +844,18 @@ void gnutls_certificate_set_retrieve_function2
  *
  * The callback function should set the certificate and OCSP response
  * list to be sent, and return 0 on success. If no certificates are available,
- * the @pcert_length and @ocsp_length should be set to zero. The return
+ * the @certs_length and @ocsp_length should be set to zero. The return
  * value (-1) indicates error and the handshake will be terminated. If both
  * certificates are set in the credentials and a callback is available, the
  * callback takes predence.
  *
+ * Raw public-keys:
+ * In case raw public-keys are negotiated as certificate type, certificates
+ * that would normally hold the public-key material are not available. In that case,
+ * @certs contains an allocated list with only the public key. Since there is no
+ * certificate, there is also no certificate status. Therefore, OCSP information
+ * should not be set.
+ *
  * Since: 3.6.3
  **/
 void gnutls_certificate_set_retrieve_function3
