@@ -398,6 +398,12 @@ int _gnutls_fips_perform_self_checks2(void)
 		goto error;
 	}
 
+	ret = gnutls_mac_self_test(0, GNUTLS_MAC_AES_CMAC_256);
+	if (ret < 0) {
+		gnutls_assert();
+		goto error;
+	}
+
 	/* PK */
 	ret = gnutls_pk_self_test(0, GNUTLS_PK_RSA);
 	if (ret < 0) {
@@ -423,6 +429,27 @@ int _gnutls_fips_perform_self_checks2(void)
 		goto error;
 	}
 
+	/* HKDF */
+	ret = gnutls_hkdf_self_test(0, GNUTLS_MAC_SHA256);
+	if (ret < 0) {
+		gnutls_assert();
+		goto error;
+	}
+
+	/* PBKDF2 */
+	ret = gnutls_pbkdf2_self_test(0, GNUTLS_MAC_SHA256);
+	if (ret < 0) {
+		gnutls_assert();
+		goto error;
+	}
+
+	/* TLS-PRF */
+	ret = gnutls_tlsprf_self_test(0, GNUTLS_MAC_SHA256);
+	if (ret < 0) {
+		gnutls_assert();
+		goto error;
+	}
+
 	if (_gnutls_rnd_ops.self_test == NULL) {
 		gnutls_assert();
 		goto error;
@@ -491,8 +518,17 @@ unsigned gnutls_fips140_mode_enabled(void)
 #ifdef ENABLE_FIPS140
 	unsigned ret = _gnutls_fips_mode_enabled();
 
-	if (ret > GNUTLS_FIPS140_DISABLED)
+	if (ret > GNUTLS_FIPS140_DISABLED) {
+		/* If the previous run of selftests has failed, return as if
+		 * the FIPS mode is disabled. We could use HAVE_LIB_ERROR, if
+		 * we can assume that all the selftests run atomically from
+		 * the ELF constructor.
+		 */
+		if (_gnutls_get_lib_state() == LIB_STATE_ERROR)
+			return 0;
+
 		return ret;
+	}
 #endif
 	return 0;
 }
