@@ -17,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -33,6 +33,7 @@
 #include <locks.h>
 #include <system.h>
 #include <accelerated/cryptodev.h>
+#include <accelerated/afalg.h>
 #include <accelerated/accelerated.h>
 #include <fips.h>
 #include <atfork.h>
@@ -67,18 +68,20 @@ inline static int _gnutls_global_init_skip(void)
 #endif
 
 /* created by asn1c */
-extern const ASN1_ARRAY_TYPE gnutls_asn1_tab[];
-extern const ASN1_ARRAY_TYPE pkix_asn1_tab[];
+extern const asn1_static_node gnutls_asn1_tab[];
+extern const asn1_static_node pkix_asn1_tab[];
 void *_gnutls_file_mutex;
 void *_gnutls_pkcs11_mutex;
 
-ASN1_TYPE _gnutls_pkix1_asn = ASN1_TYPE_EMPTY;
-ASN1_TYPE _gnutls_gnutls_asn = ASN1_TYPE_EMPTY;
+asn1_node _gnutls_pkix1_asn = NULL;
+asn1_node _gnutls_gnutls_asn = NULL;
 
 gnutls_log_func _gnutls_log_func = NULL;
 gnutls_audit_log_func _gnutls_audit_log_func = NULL;
 int _gnutls_log_level = 0;	/* default log level */
 
+unsigned int _gnutls_global_version = GNUTLS_VERSION_NUMBER;
+
 static int _gnutls_global_init(unsigned constructor);
 static void _gnutls_global_deinit(unsigned destructor);
 
@@ -196,7 +199,7 @@ static int _gnutls_init_ret = 0;
  *
  * Since GnuTLS 3.3.0 this function is no longer necessary to be explicitly
  * called. To disable the implicit call (in a library constructor) of this
- * function set the environment variable %GNUTLS_NO_EXPLICIT_INIT to 1.
+ * function set the environment variable %GNUTLS_NO_IMPLICIT_INIT to 1.
  *
  * This function performs any required precalculations, detects
  * the supported CPU capabilities and initializes the underlying
@@ -209,7 +212,7 @@ static int _gnutls_init_ret = 0;
  * called as many times as gnutls_global_init().  This is useful when
  * GnuTLS is used by more than one library in an application.  This
  * function can be called many times, but will only do something the
- * first time.
+ * first time. It is thread safe since GnuTLS 3.3.0.
  *
  * A subsequent call of this function if the initial has failed will
  * return the same error code.
@@ -234,16 +237,6 @@ static int _gnutls_global_init(unsigned constructor)
 
 	_gnutls_init++;
 	if (_gnutls_init > 1) {
-		if (_gnutls_init == 2 && _gnutls_init_ret == 0) {
-			/* some applications may close the urandom fd 
-			 * before calling gnutls_global_init(). in that
-			 * case reopen it */
-			ret = _gnutls_rnd_check();
-			if (ret < 0) {
-				gnutls_assert();
-				goto out;
-			}
-		}
 		ret = _gnutls_init_ret;
 		goto out;
 	}
@@ -287,7 +280,7 @@ static int _gnutls_global_init(unsigned constructor)
 		goto out;
 	}
 
-	_gnutls_pkix1_asn = ASN1_TYPE_EMPTY;
+	_gnutls_pkix1_asn = NULL;
 	res = asn1_array2tree(pkix_asn1_tab, &_gnutls_pkix1_asn, NULL);
 	if (res != ASN1_SUCCESS) {
 		gnutls_assert();
@@ -366,7 +359,7 @@ static int _gnutls_global_init(unsigned constructor)
 
 	_gnutls_register_accel_crypto();
 	_gnutls_cryptodev_init();
-	_gnutls_load_system_priorities();
+	_gnutls_afalg_init();
 
 #ifdef ENABLE_FIPS140
 	/* These self tests are performed on the overridden algorithms
@@ -383,6 +376,7 @@ static int _gnutls_global_init(unsigned constructor)
 		_gnutls_fips_mode_reset_zombie();
 	}
 #endif
+	_gnutls_load_system_priorities();
 	_gnutls_switch_lib_state(LIB_STATE_OPERATIONAL);
 	ret = 0;
 
@@ -493,14 +487,22 @@ const char *gnutls_check_version(const char *req_version)
 
 static void _CONSTRUCTOR lib_init(void)
 {
-int ret;
-const char *e;
+	int ret;
+	const char *e;
 
 	if (_gnutls_global_init_skip() != 0)
 		return;
 
+	e = secure_getenv("GNUTLS_NO_IMPLICIT_INIT");
+	if (e != NULL) {
+		ret = atoi(e);
+		if (ret == 1)
+			return;
+	}
+
 	e = secure_getenv("GNUTLS_NO_EXPLICIT_INIT");
 	if (e != NULL) {
+		_gnutls_debug_log("GNUTLS_NO_EXPLICIT_INIT is deprecated; use GNUTLS_NO_IMPLICIT_INIT\n");
 		ret = atoi(e);
 		if (ret == 1)
 			return;
@@ -515,14 +517,23 @@ const char *e;
 
 static void _DESTRUCTOR lib_deinit(void)
 {
+	int ret;
 	const char *e;
 
 	if (_gnutls_global_init_skip() != 0)
 		return;
 
+	e = secure_getenv("GNUTLS_NO_IMPLICIT_INIT");
+	if (e != NULL) {
+		ret = atoi(e);
+		if (ret == 1)
+			return;
+	}
+
 	e = secure_getenv("GNUTLS_NO_EXPLICIT_INIT");
 	if (e != NULL) {
-		int ret = atoi(e);
+		_gnutls_debug_log("GNUTLS_NO_EXPLICIT_INIT is deprecated; use GNUTLS_NO_IMPLICIT_INIT\n");
+		ret = atoi(e);
 		if (ret == 1)
 			return;
 	}
