@@ -54,7 +54,7 @@ static int generate_normal_master(gnutls_session_t session,
 
 int _gnutls_generate_master(gnutls_session_t session, int keep_premaster)
 {
-	if (session->internals.resumed == RESUME_FALSE)
+	if (!session->internals.resumed)
 		return generate_normal_master(session, &session->key.key,
 					      keep_premaster);
 	else if (session->internals.premaster_set) {
@@ -132,6 +132,10 @@ _gnutls_nss_keylog_func(gnutls_session_t session,
 	return 0;
 }
 
+/* GCC analyzer doesn't like static FILE pointer */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wanalyzer-file-leak"
+
 void _gnutls_nss_keylog_write(gnutls_session_t session,
 			      const char *label,
 			      const uint8_t *secret, size_t secret_size)
@@ -172,6 +176,8 @@ void _gnutls_nss_keylog_deinit(void)
 	}
 }
 
+#pragma GCC diagnostic pop
+
 /* here we generate the TLS Master secret.
  */
 static int
