@@ -25,14 +25,16 @@
 
 #include <config.h>
 
-/* this realloc function will return ptr if size==0, and
- * will free the ptr if the new allocation failed.
+/* These realloc functions will return ptr if size==0, and will free
+ * the ptr if the new allocation failed.
  */
 void *gnutls_realloc_fast(void *ptr, size_t size);
+void *_gnutls_reallocarray_fast(void *ptr, size_t nmemb, size_t size);
 
-void *_gnutls_calloc(size_t nmemb, size_t size);
 char *_gnutls_strdup(const char *);
 
+void *_gnutls_reallocarray(void *, size_t, size_t);
+
 unsigned _gnutls_mem_is_zero(const uint8_t *ptr, unsigned size);
 
 #define zrelease_mpi_key(mpi) if (*mpi!=NULL) { \
