@@ -271,6 +271,9 @@ const asn1_static_node pkix_asn1_tab[] = {
   { NULL, 1048586, "16"},
   { "pkcs-5-aes256-CBC-params", 1612709895, NULL },
   { NULL, 1048586, "16"},
+  { "Gost28147-89-Parameters", 1610612741, NULL },
+  { "iv", 1073741831, NULL },
+  { "encryptionParamSet", 12, NULL },
   { "pkcs-5-PBE-params", 1610612741, NULL },
   { "salt", 1073741831, NULL },
   { "iterationCount", 3, NULL },
