@@ -989,6 +989,10 @@ static void enable_server_precedence(gnutls_priority_t c)
 {
 	c->server_precedence = 1;
 }
+static void disable_tls13_compat_mode(gnutls_priority_t c)
+{
+	c->tls13_compat_mode = false;
+}
 static void dummy_func(gnutls_priority_t c)
 {
 }
@@ -1802,7 +1806,7 @@ static int set_ciphersuite_list(gnutls_priority_t priority_cache)
 
 /**
  * gnutls_priority_init2:
- * @priority_cache: is a #gnutls_prioritity_t type.
+ * @priority_cache: is a #gnutls_priority_t type.
  * @priorities: is a string describing priorities (may be %NULL)
  * @err_pos: In case of an error this will have the position in the string the error occurred
  * @flags: zero or %GNUTLS_PRIORITY_INIT_DEF_APPEND
@@ -1957,7 +1961,7 @@ gnutls_priority_init2(gnutls_priority_t * priority_cache,
 
 /**
  * gnutls_priority_init:
- * @priority_cache: is a #gnutls_prioritity_t type.
+ * @priority_cache: is a #gnutls_priority_t type.
  * @priorities: is a string describing priorities (may be %NULL)
  * @err_pos: In case of an error this will have the position in the string the error occurred
  *
@@ -2005,6 +2009,11 @@ gnutls_priority_init(gnutls_priority_t * priority_cache,
 	 * when we make it the default.
 	 */
 	(*priority_cache)->sr = SR_PARTIAL;
+	/* For now TLS 1.3 middlebox compatibility mode is enabled by default.
+	 * This will eventually be disabled by default and moved to the %COMPAT
+	 * setting.
+	 */
+	(*priority_cache)->tls13_compat_mode = true;
 	(*priority_cache)->min_record_version = 1;
 	gnutls_atomic_init(&(*priority_cache)->usage_cnt);
 
@@ -2288,7 +2297,7 @@ gnutls_priority_init(gnutls_priority_t * priority_cache,
 
 /**
  * gnutls_priority_deinit:
- * @priority_cache: is a #gnutls_prioritity_t type.
+ * @priority_cache: is a #gnutls_priority_t type.
  *
  * Deinitializes the priority cache.
  **/
@@ -2469,7 +2478,7 @@ int gnutls_set_default_priority_append(gnutls_session_t session,
 
 /**
  * gnutls_priority_ecc_curve_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available elliptic curves in the priority
@@ -2504,7 +2513,7 @@ gnutls_priority_ecc_curve_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_group_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available groups in the priority
@@ -2527,7 +2536,7 @@ gnutls_priority_group_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_kx_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available key exchange methods in the priority
@@ -2549,7 +2558,7 @@ gnutls_priority_kx_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_cipher_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available ciphers in the priority
@@ -2571,7 +2580,7 @@ gnutls_priority_cipher_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_mac_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available MAC algorithms in the priority
@@ -2593,7 +2602,7 @@ gnutls_priority_mac_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_compression_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available compression method in the priority
@@ -2614,7 +2623,7 @@ gnutls_priority_compression_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_protocol_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available TLS version numbers in the priority
@@ -2636,7 +2645,7 @@ gnutls_priority_protocol_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_sign_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available signature algorithms in the priority
@@ -2658,7 +2667,7 @@ gnutls_priority_sign_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_certificate_type_list:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list
  *
  * Get a list of available certificate types in the priority
@@ -2685,7 +2694,7 @@ gnutls_priority_certificate_type_list(gnutls_priority_t pcache,
 
 /**
  * gnutls_priority_certificate_type_list2:
- * @pcache: is a #gnutls_prioritity_t type.
+ * @pcache: is a #gnutls_priority_t type.
  * @list: will point to an integer list.
  * @target: is a #gnutls_ctype_target_t type. Valid arguments are
  *   GNUTLS_CTYPE_CLIENT and GNUTLS_CTYPE_SERVER
