@@ -369,7 +369,7 @@ gnutls_pubkey_parse_ecc_eddsa_params(const gnutls_datum_t *parameters,
 				     gnutls_ecc_curve_t *outcurve)
 {
 	gnutls_ecc_curve_t curve = GNUTLS_ECC_CURVE_INVALID;
-	ASN1_TYPE asn1 = ASN1_TYPE_EMPTY;
+	asn1_node asn1 = NULL;
 	unsigned int etype = ASN1_ETYPE_INVALID;
 	char str[MAX_OID_SIZE];
 	int str_size;
@@ -622,7 +622,7 @@ gnutls_pubkey_export(gnutls_pubkey_t key,
 		     size_t * output_data_size)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -686,7 +686,7 @@ gnutls_pubkey_export2(gnutls_pubkey_t key,
 		      gnutls_x509_crt_fmt_t format, gnutls_datum_t * out)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -1223,7 +1223,7 @@ gnutls_pubkey_import(gnutls_pubkey_t key,
 {
 	int result = 0, need_free = 0;
 	gnutls_datum_t _data;
-	ASN1_TYPE spk;
+	asn1_node spk;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -2092,10 +2092,16 @@ int _gnutls_pubkey_compatible_with_sig(gnutls_session_t session,
 	unsigned int sig_hash_size;
 	const mac_entry_st *me;
 	const gnutls_sign_entry_st *se;
+	int ret;
 
 	se = _gnutls_sign_to_entry(sign);
-	if (se == NULL && _gnutls_version_has_selectable_sighash(ver))
+	if (se != NULL) {
+		ret = pubkey_supports_sig(pubkey, se);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+	} else if (_gnutls_version_has_selectable_sighash(ver)) {
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+	}
 
 	if (pubkey->params.algo == GNUTLS_PK_DSA) {
 		me = _gnutls_dsa_q_to_hash(&pubkey->params, &hash_size);
@@ -2158,9 +2164,6 @@ int _gnutls_pubkey_compatible_with_sig(gnutls_session_t session,
 		}
 	}
 
-	if (se != NULL)
-		return pubkey_supports_sig(pubkey, se);
-
 	return 0;
 }
 
@@ -2578,15 +2581,17 @@ int gnutls_pubkey_verify_params(gnutls_pubkey_t key)
 int
 gnutls_pubkey_get_spki(gnutls_pubkey_t pubkey, gnutls_x509_spki_t spki, unsigned int flags)
 {
+	gnutls_x509_spki_t p = &pubkey->params.spki;
+
 	if (pubkey == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	if (pubkey->params.spki.pk == GNUTLS_PK_UNKNOWN)
+	if (p->pk == GNUTLS_PK_UNKNOWN)
 		return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
 
-	memcpy(spki, &pubkey->params.spki, sizeof(gnutls_x509_spki_st));
+	memcpy(spki, p, sizeof(gnutls_x509_spki_st));
 
 	return 0;
 }
