@@ -341,7 +341,7 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 
 inline static void session_unresumable(gnutls_session_t session)
 {
-	session->internals.resumable = RESUME_FALSE;
+	session->internals.resumable = false;
 }
 
 /* returns 0 if session is valid
@@ -2098,6 +2098,10 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
 	if (session->security_parameters.entity != GNUTLS_CLIENT)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
+	if (data_size == 0) {
+		return 0;
+	}
+
 	if (xsum(session->internals.
 		 early_data_presend_buffer.length,
 		 data_size) >
@@ -2111,6 +2115,8 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
+	session->internals.flags |= GNUTLS_ENABLE_EARLY_DATA;
+
 	return ret;
 }
 
@@ -2120,7 +2126,7 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
  * @data: the buffer that the data will be read into
  * @data_size: the number of requested bytes
  *
- * This function can be used by a searver to retrieve data sent early
+ * This function can be used by a server to retrieve data sent early
  * in the handshake processes when resuming a session.  This is used
  * to implement a zero-roundtrip (0-RTT) mode.  It has the same
  * semantics as gnutls_record_recv().
