@@ -166,7 +166,7 @@ gnutls_session_get_data2(gnutls_session_t session, gnutls_datum_t *data)
 		}
 	}
 
-	if (session->internals.resumable == RESUME_FALSE)
+	if (!session->internals.resumable)
 		return GNUTLS_E_INVALID_SESSION;
 
 	ret = _gnutls_session_pack(session, data);
