@@ -238,14 +238,16 @@ _gnutls_read_client_hello_v2(gnutls_session_t session, uint8_t * data,
 		       session->security_parameters.client_random,
 		       GNUTLS_RANDOM_SIZE);
 
-		session->internals.resumed = RESUME_TRUE;
+		session->internals.resumed = true;
 		return 0;
 	} else {
-		_gnutls_generate_session_id(session->security_parameters.
-					    session_id,
-					    &session->security_parameters.
-					    session_id_size);
-		session->internals.resumed = RESUME_FALSE;
+		ret = _gnutls_generate_session_id(
+			session->security_parameters.session_id,
+			&session->security_parameters.session_id_size);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+
+		session->internals.resumed = false;
 	}
 
 	return sret;
