@@ -91,6 +91,40 @@ gnutls_cipher_algorithm_t gnutls_cipher_get(gnutls_session_t session)
 	return record_params->cipher->id;
 }
 
+/**
+ * gnutls_early_cipher_get:
+ * @session: is a #gnutls_session_t type.
+ *
+ * Get the cipher algorithm used for encrypting early data.
+ *
+ * Returns: the cipher used for early data, a
+ *   #gnutls_cipher_algorithm_t type.
+ *
+ * Since: 3.7.2
+ **/
+gnutls_cipher_algorithm_t gnutls_early_cipher_get(gnutls_session_t session)
+{
+	const cipher_entry_st *ce;
+
+	if (!(session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT)) {
+		return gnutls_assert_val(GNUTLS_CIPHER_UNKNOWN);
+	}
+
+	if (unlikely(session->internals.
+		     resumed_security_parameters.cs == NULL)) {
+		return gnutls_assert_val(GNUTLS_CIPHER_UNKNOWN);
+	}
+
+	ce = cipher_to_entry(session->internals.
+			     resumed_security_parameters.
+			     cs->block_algorithm);
+	if (unlikely(ce == NULL)) {
+		return gnutls_assert_val(GNUTLS_CIPHER_UNKNOWN);
+	}
+
+	return ce->id;
+}
+
 /**
  * gnutls_certificate_type_get:
  * @session: is a #gnutls_session_t type.
@@ -259,6 +293,39 @@ gnutls_prf_hash_get(const gnutls_session_t session)
 	return (gnutls_digest_algorithm_t)session->security_parameters.prf->id;
 }
 
+/**
+ * gnutls_early_prf_hash_get:
+ * @session: is a #gnutls_session_t type.
+ *
+ * Get the hash algorithm used as a PRF to derive keys for encrypting
+ * early data in TLS 1.3.
+ *
+ * Returns: the hash algorithm used for early data, a
+ *    #gnutls_digest_algorithm_t value.
+ *
+ * Since: 3.7.2
+ **/
+gnutls_digest_algorithm_t
+gnutls_early_prf_hash_get(const gnutls_session_t session)
+{
+	if (!(session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT)) {
+		return gnutls_assert_val(GNUTLS_DIG_UNKNOWN);
+	}
+
+	if (unlikely(session->internals.
+		     resumed_security_parameters.prf == NULL)) {
+		return gnutls_assert_val(GNUTLS_DIG_UNKNOWN);
+	}
+
+	if (unlikely(session->internals.
+		     resumed_security_parameters.prf->id >= GNUTLS_MAC_AEAD)) {
+		return gnutls_assert_val(GNUTLS_DIG_UNKNOWN);
+	}
+
+	return (gnutls_digest_algorithm_t)session->internals.
+		resumed_security_parameters.prf->id;
+}
+
 void reset_binders(gnutls_session_t session)
 {
 	_gnutls_free_temp_key_datum(&session->key.binders[0].psk);
@@ -311,8 +378,8 @@ static bool _gnutls_has_cert_credentials(gnutls_session_t session,
  * and in some cases a matching certificate exists. A check for
  * the latter can be toggled via the parameter @check_credentials.
  */
-int
-_gnutls_session_cert_type_supported(gnutls_session_t session,
+bool
+_gnutls_session_is_cert_type_supported(gnutls_session_t session,
 				    gnutls_certificate_type_t cert_type,
 				    bool check_credentials,
 				    gnutls_ctype_target_t target)
@@ -419,7 +486,7 @@ static void handshake_internal_state_clear1(gnutls_session_t session)
 	session->internals.last_handshake_in = -1;
 	session->internals.last_handshake_out = -1;
 
-	session->internals.resumable = RESUME_TRUE;
+	session->internals.resumable = true;
 
 	session->internals.handshake_suspicious_loops = 0;
 	session->internals.dtls.hsk_read_seq = 0;
@@ -588,16 +655,12 @@ int gnutls_init(gnutls_session_t * session, unsigned int flags)
 #endif
 	handshake_internal_state_clear1(*session);
 
-#ifdef HAVE_WRITEV
 #ifdef MSG_NOSIGNAL
 	if (flags & GNUTLS_NO_SIGNAL)
 		gnutls_transport_set_vec_push_function(*session, system_writev_nosignal);
 	else
 #endif
 		gnutls_transport_set_vec_push_function(*session, system_writev);
-#else
-	gnutls_transport_set_push_function(*session, system_write);
-#endif
 	(*session)->internals.pull_timeout_func = gnutls_system_recv_timeout;
 	(*session)->internals.pull_func = system_read;
 	(*session)->internals.errno_func = system_errno;
@@ -640,13 +703,6 @@ int gnutls_init(gnutls_session_t * session, unsigned int flags)
 	return 0;
 }
 
-/* returns RESUME_FALSE or RESUME_TRUE.
- */
-int _gnutls_session_is_resumable(gnutls_session_t session)
-{
-	return session->internals.resumable;
-}
-
 
 /**
  * gnutls_deinit:
@@ -988,9 +1044,9 @@ int gnutls_session_is_resumed(gnutls_session_t session)
 {
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
 		const version_entry_st *ver = get_version(session);
-		if (ver && ver->tls13_sem &&
-		    session->internals.resumed != RESUME_FALSE)
-			return 1;
+		if (ver && ver->tls13_sem) {
+			return session->internals.resumed;
+		}
 
 		if (session->security_parameters.session_id_size > 0 &&
 		    session->security_parameters.session_id_size ==
@@ -1004,7 +1060,7 @@ int gnutls_session_is_resumed(gnutls_session_t session)
 			      session_id_size) == 0)
 			return 1;
 	} else {
-		if (session->internals.resumed != RESUME_FALSE)
+		if (session->internals.resumed)
 			return 1;
 	}
 
@@ -1265,20 +1321,117 @@ gnutls_session_channel_binding(gnutls_session_t session,
 			       gnutls_channel_binding_t cbtype,
 			       gnutls_datum_t * cb)
 {
-	if (cbtype != GNUTLS_CB_TLS_UNIQUE)
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
-
 	if (!session->internals.initial_negotiation_completed)
 		return GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE;
 
-	cb->size = session->internals.cb_tls_unique_len;
-	cb->data = gnutls_malloc(cb->size);
-	if (cb->data == NULL)
-		return GNUTLS_E_MEMORY_ERROR;
+	if (cbtype == GNUTLS_CB_TLS_UNIQUE) {
+		const version_entry_st *ver = get_version(session);
+		if (unlikely(ver == NULL || ver->tls13_sem))
+			return GNUTLS_E_INVALID_REQUEST;
 
-	memcpy(cb->data, session->internals.cb_tls_unique, cb->size);
+		cb->size = session->internals.cb_tls_unique_len;
+		cb->data = gnutls_malloc(cb->size);
+		if (cb->data == NULL)
+			return GNUTLS_E_MEMORY_ERROR;
 
-	return 0;
+		memcpy(cb->data, session->internals.cb_tls_unique, cb->size);
+
+		return 0;
+	}
+
+	if (cbtype == GNUTLS_CB_TLS_SERVER_END_POINT) {
+		const gnutls_datum_t *ders;
+		unsigned int num_certs = 1;
+		int ret;
+		size_t rlen;
+		gnutls_x509_crt_t cert;
+		gnutls_digest_algorithm_t algo;
+
+		/* Only X509 certificates are supported for this binding type */
+		ret = gnutls_certificate_type_get (session);
+		if (ret != GNUTLS_CRT_X509)
+			return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+
+		if (session->security_parameters.entity == GNUTLS_CLIENT)
+			ders = gnutls_certificate_get_peers (session, &num_certs);
+		else
+			ders = gnutls_certificate_get_ours (session);
+
+		/* Previous check indicated we have x509 but you never know */
+		if (!ders || num_certs == 0)
+			return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+
+		ret = gnutls_x509_crt_list_import (&cert, &num_certs, ders,
+				GNUTLS_X509_FMT_DER, 0);
+		/* Again, this is not supposed to happen (normally) */
+		if (ret < 0 || num_certs == 0)
+			return GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE;
+
+		/* Obtain signature algorithm used by certificate */
+		ret = gnutls_x509_crt_get_signature_algorithm (cert);
+		if (ret < 0 || ret == GNUTLS_SIGN_UNKNOWN)
+			return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+
+		/* obtain hash function from signature and normalize it */
+		algo = gnutls_sign_get_hash_algorithm (ret);
+		switch (algo) {
+		case GNUTLS_DIG_MD5:
+		case GNUTLS_DIG_SHA1:
+			algo = GNUTLS_DIG_SHA256;
+			break;
+		case GNUTLS_DIG_UNKNOWN:
+		case GNUTLS_DIG_NULL:
+		case GNUTLS_DIG_MD5_SHA1:
+			/* double hashing not supported either */
+			gnutls_x509_crt_deinit (cert);
+			return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		default:
+			/* no-op */
+			algo = algo;
+		}
+
+		/* preallocate 512 bits buffer as maximum supported digest */
+		rlen = MAX_HASH_SIZE;
+		cb->data = gnutls_malloc(rlen);
+		if (cb->data == NULL) {
+			gnutls_x509_crt_deinit (cert);
+			return GNUTLS_E_MEMORY_ERROR;
+		}
+
+		ret = gnutls_x509_crt_get_fingerprint (cert, algo, cb->data,
+				&rlen);
+		if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER) {
+			cb->data = gnutls_realloc_fast (cb->data, cb->size);
+			if (cb->data == NULL) {
+				gnutls_x509_crt_deinit (cert);
+				return GNUTLS_E_MEMORY_ERROR;
+			}
+			ret = gnutls_x509_crt_get_fingerprint (cert, algo,
+					cb->data, &rlen);
+		}
+		cb->size = rlen;
+		gnutls_x509_crt_deinit (cert);
+		return ret;
+	}
+
+	if (cbtype == GNUTLS_CB_TLS_EXPORTER) {
+#define RFC5705_LABEL_DATA "EXPORTER-Channel-Binding"
+#define RFC5705_LABEL_LEN 24
+#define EXPORTER_CTX_DATA ""
+#define EXPORTER_CTX_LEN 0
+
+		cb->size = 32;
+		cb->data = gnutls_malloc(cb->size);
+		if (cb->data == NULL)
+			return GNUTLS_E_MEMORY_ERROR;
+
+		return gnutls_prf_rfc5705 (session,
+				RFC5705_LABEL_LEN, RFC5705_LABEL_DATA,
+				EXPORTER_CTX_LEN, EXPORTER_CTX_DATA,
+				cb->size, (char *) cb->data);
+	}
+
+	return GNUTLS_E_UNIMPLEMENTED_FEATURE;
 }
 
 /**
