@@ -17,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -29,7 +29,6 @@
 #include <c-ctype.h>
 #include <intprops.h>
 #include <nettle/base64.h>
-#include "vasprintf.h"
 #include "extras/hex.h"
 
 /* These functions are like strcat, strcpy. They only
@@ -81,22 +80,13 @@ void _gnutls_buffer_clear(gnutls_buffer_st * str)
 		return;
 	gnutls_free(str->allocd);
 
-	str->data = str->allocd = NULL;
+	str->data = NULL;
 	str->max_length = 0;
 	str->length = 0;
 }
 
 #define MIN_CHUNK 1024
 
-static void align_allocd_with_data(gnutls_buffer_st * dest)
-{
-	assert(dest->allocd != NULL);
-	assert(dest->data != NULL);
-	if (dest->length)
-		memmove(dest->allocd, dest->data, dest->length);
-	dest->data = dest->allocd;
-}
-
 /**
  * gnutls_buffer_append_data:
  * @dest: the buffer to append to
@@ -114,7 +104,7 @@ gnutls_buffer_append_data(gnutls_buffer_t dest, const void *data,
 			   size_t data_size)
 {
 	size_t const tot_len = data_size + dest->length;
-	size_t const unused = MEMSUB(dest->data, dest->allocd);
+	int ret;
 
 	if (unlikely(dest->data != NULL && dest->allocd == NULL))
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
@@ -127,25 +117,9 @@ gnutls_buffer_append_data(gnutls_buffer_t dest, const void *data,
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
 
-	if (dest->max_length >= tot_len) {
-
-		if (dest->max_length - unused <= tot_len) {
-			align_allocd_with_data(dest);
-		}
-	} else {
-		size_t const new_len =
-		    MAX(data_size, MIN_CHUNK) + MAX(dest->max_length,
-						    MIN_CHUNK);
-
-		dest->allocd = gnutls_realloc_fast(dest->allocd, new_len);
-		if (dest->allocd == NULL) {
-			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
-		}
-		dest->max_length = new_len;
-		dest->data = dest->allocd + unused;
-
-		align_allocd_with_data(dest);
+	ret = _gnutls_buffer_resize(dest, tot_len);
+	if (ret < 0) {
+		return ret;
 	}
 	assert(dest->data != NULL);
 
@@ -155,6 +129,45 @@ gnutls_buffer_append_data(gnutls_buffer_t dest, const void *data,
 	return 0;
 }
 
+#ifdef AGGRESSIVE_REALLOC
+
+/* Use a simpler logic for reallocation; i.e., always call
+ * gnutls_realloc_fast() and do not reclaim the no-longer-used
+ * area which has been removed from the beginning of buffer
+ * with _gnutls_buffer_pop_datum().  This helps hit more
+ * issues when running under valgrind.
+ */
+int _gnutls_buffer_resize(gnutls_buffer_st * dest, size_t new_size)
+{
+	size_t unused;
+
+	if (unlikely(dest->data != NULL && dest->allocd == NULL))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	unused = MEMSUB(dest->data, dest->allocd);
+	dest->allocd =
+	    gnutls_realloc_fast(dest->allocd, new_size + unused);
+	if (dest->allocd == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+	dest->max_length = new_size + unused;
+	dest->data = dest->allocd + unused;
+
+	return 0;
+}
+
+#else
+
+static void align_allocd_with_data(gnutls_buffer_st * dest)
+{
+	assert(dest->allocd != NULL);
+	assert(dest->data != NULL);
+	if (dest->length)
+		memmove(dest->allocd, dest->data, dest->length);
+	dest->data = dest->allocd;
+}
+
 int _gnutls_buffer_resize(gnutls_buffer_st * dest, size_t new_size)
 {
 	if (unlikely(dest->data != NULL && dest->allocd == NULL))
@@ -188,6 +201,8 @@ int _gnutls_buffer_resize(gnutls_buffer_st * dest, size_t new_size)
 	}
 }
 
+#endif
+
 /* Appends the provided string. The null termination byte is appended
  * but not included in length.
  */
@@ -224,7 +239,7 @@ _gnutls_buffer_pop_datum(gnutls_buffer_st * str, gnutls_datum_t * data,
 	str->data += req_size;
 	str->length -= req_size;
 
-	/* if string becomes empty start from begining */
+	/* if string becomes empty start from beginning */
 	if (str->length == 0) {
 		str->data = str->allocd;
 	}
@@ -232,7 +247,7 @@ _gnutls_buffer_pop_datum(gnutls_buffer_st * str, gnutls_datum_t * data,
 	return;
 }
 
-/* converts the buffer to a datum if possible. After this call 
+/* converts the buffer to a datum if possible. After this call
  * (failed or not) the buffer should be considered deinitialized.
  */
 int _gnutls_buffer_to_datum(gnutls_buffer_st * str, gnutls_datum_t * data, unsigned is_str)
@@ -400,20 +415,17 @@ int _gnutls_buffer_unescape(gnutls_buffer_st * dest)
 
 	while (pos < dest->length) {
 		if (dest->data[pos] == '%') {
-			char b[3];
-			unsigned int u;
-			unsigned char x;
+			if (pos + 1 < dest->length && dest->data[pos + 1] == '%') {
+				// %% -> %
+				_gnutls_buffer_delete_data(dest, pos, 1);
+			} else if (pos + 2 < dest->length && c_isxdigit(dest->data[pos + 1]) && c_isxdigit(dest->data[pos + 2])) {
+				unsigned char x;
 
-			b[0] = dest->data[pos + 1];
-			b[1] = dest->data[pos + 2];
-			b[2] = 0;
+				hex_decode((char *) dest->data + pos + 1, 2, &x, 1);
 
-			sscanf(b, "%02x", &u);
-
-			x = u;
-
-			_gnutls_buffer_delete_data(dest, pos, 3);
-			_gnutls_buffer_insert_data(dest, pos, &x, 1);
+				_gnutls_buffer_delete_data(dest, pos, 3);
+				_gnutls_buffer_insert_data(dest, pos, &x, 1);
+			}
 		}
 		pos++;
 	}
@@ -657,7 +669,7 @@ gnutls_hex_encode2(const gnutls_datum_t * data, gnutls_datum_t *result)
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	ret = hex_encode((char*)data->data, data->size, (char*)result->data, size); 
+	ret = hex_encode((char*)data->data, data->size, (char*)result->data, size);
 	if (ret == 0) {
 		gnutls_free(result->data);
 		return gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
