@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -34,12 +34,12 @@
 #include <datum.h>
 
 int
-_gnutls_get_extension(ASN1_TYPE asn, const char *root,
+_gnutls_get_extension(asn1_node asn, const char *root,
 	      const char *extension_id, int indx,
 	      gnutls_datum_t * ret, unsigned int *_critical)
 {
 	int k, result, len;
-	char name[ASN1_MAX_NAME_SIZE], name2[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE], name2[MAX_NAME_SIZE];
 	char str_critical[10];
 	int critical = 0;
 	char extnID[MAX_OID_SIZE];
@@ -62,7 +62,6 @@ _gnutls_get_extension(ASN1_TYPE asn, const char *root,
 		result = asn1_read_value(asn, name2, extnID, &len);
 
 		if (result == ASN1_ELEMENT_NOT_FOUND) {
-			gnutls_assert();
 			break;
 		} else if (result != ASN1_SUCCESS) {
 			gnutls_assert();
@@ -132,10 +131,10 @@ _gnutls_get_extension(ASN1_TYPE asn, const char *root,
 }
 
 static int
-get_indx_extension(ASN1_TYPE asn, const char *root,
+get_indx_extension(asn1_node asn, const char *root,
 	      int indx, gnutls_datum_t * out)
 {
-	char name[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE];
 	int ret;
 
 	out->data = NULL;
@@ -227,11 +226,11 @@ gnutls_x509_crl_get_extension_data2(gnutls_x509_crl_t crl,
  * If you have passed the last extension, GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE will
  * be returned.
  */
-static int get_extension_oid(ASN1_TYPE asn, const char *root,
+static int get_extension_oid(asn1_node asn, const char *root,
 		  unsigned indx, void *oid, size_t * sizeof_oid)
 {
 	int k, result, len;
-	char name[ASN1_MAX_NAME_SIZE], name2[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE], name2[MAX_NAME_SIZE];
 	char extnID[MAX_OID_SIZE];
 	unsigned indx_counter = 0;
 
@@ -314,12 +313,12 @@ _gnutls_x509_crl_get_extension_oid(gnutls_x509_crl_t crl,
  * Critical will be either 0 or 1.
  */
 static int
-add_extension(ASN1_TYPE asn, const char *root, const char *extension_id,
+add_extension(asn1_node asn, const char *root, const char *extension_id,
 	      const gnutls_datum_t * ext_data, unsigned int critical)
 {
 	int result;
 	const char *str;
-	char name[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE];
 
 	snprintf(name, sizeof(name), "%s", root);
 
@@ -376,10 +375,10 @@ add_extension(ASN1_TYPE asn, const char *root, const char *extension_id,
  * index here starts from one.
  */
 static int
-overwrite_extension(ASN1_TYPE asn, const char *root, unsigned int indx,
+overwrite_extension(asn1_node asn, const char *root, unsigned int indx,
 		    const gnutls_datum_t * ext_data, unsigned int critical)
 {
-	char name[ASN1_MAX_NAME_SIZE], name2[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE], name2[MAX_NAME_SIZE];
 	const char *str;
 	int result;
 
@@ -415,13 +414,13 @@ overwrite_extension(ASN1_TYPE asn, const char *root, unsigned int indx,
 }
 
 int
-_gnutls_set_extension(ASN1_TYPE asn, const char *root,
+_gnutls_set_extension(asn1_node asn, const char *root,
 	      const char *ext_id,
 	      const gnutls_datum_t * ext_data, unsigned int critical)
 {
 	int result = 0;
 	int k, len;
-	char name[ASN1_MAX_NAME_SIZE], name2[ASN1_MAX_NAME_SIZE];
+	char name[MAX_NAME_SIZE], name2[MAX_NAME_SIZE];
 	char extnID[MAX_OID_SIZE];
 
 	/* Find the index of the given extension.
@@ -526,7 +525,7 @@ _gnutls_x509_crq_set_extension(gnutls_x509_crq_t crq,
 	unsigned char *extensions = NULL;
 	size_t extensions_size = 0;
 	gnutls_datum_t der;
-	ASN1_TYPE c2;
+	asn1_node c2;
 	int result;
 
 	result =
@@ -615,7 +614,7 @@ _gnutls_x509_ext_extract_number(uint8_t * number,
 				size_t * _nr_size,
 				uint8_t * extnValue, int extnValueLen)
 {
-	ASN1_TYPE ext = ASN1_TYPE_EMPTY;
+	asn1_node ext = NULL;
 	int result;
 	int nr_size = *_nr_size;
 
@@ -657,7 +656,7 @@ int
 _gnutls_x509_ext_gen_number(const uint8_t * number, size_t nr_size,
 			    gnutls_datum_t * der_ext)
 {
-	ASN1_TYPE ext = ASN1_TYPE_EMPTY;
+	asn1_node ext = NULL;
 	int result;
 
 	result =
@@ -688,7 +687,7 @@ _gnutls_x509_ext_gen_number(const uint8_t * number, size_t nr_size,
 }
 
 int
-_gnutls_write_general_name(ASN1_TYPE ext, const char *ext_name,
+_gnutls_write_general_name(asn1_node ext, const char *ext_name,
 		       gnutls_x509_subject_alt_name_t type,
 		       const void *data, unsigned int data_size)
 {
@@ -716,6 +715,9 @@ _gnutls_write_general_name(ASN1_TYPE ext, const char *ext_name,
 	case GNUTLS_SAN_IPADDRESS:
 		str = "iPAddress";
 		break;
+	case GNUTLS_SAN_REGISTERED_ID:
+		str = "registeredID";
+		break;
 	default:
 		gnutls_assert();
 		return GNUTLS_E_INTERNAL_ERROR;
@@ -740,7 +742,7 @@ _gnutls_write_general_name(ASN1_TYPE ext, const char *ext_name,
 }
 
 int
-_gnutls_write_new_general_name(ASN1_TYPE ext, const char *ext_name,
+_gnutls_write_new_general_name(asn1_node ext, const char *ext_name,
 		       gnutls_x509_subject_alt_name_t type,
 		       const void *data, unsigned int data_size)
 {
@@ -771,7 +773,7 @@ _gnutls_write_new_general_name(ASN1_TYPE ext, const char *ext_name,
 }
 
 int
-_gnutls_write_new_othername(ASN1_TYPE ext, const char *ext_name,
+_gnutls_write_new_othername(asn1_node ext, const char *ext_name,
 		       const char *oid,
 		       const void *data, unsigned int data_size)
 {
@@ -904,6 +906,6 @@ _gnutls_x509_ext_gen_auth_key_id(const void *id, size_t id_size,
 	ret = 0;
 
  cleanup:
- 	gnutls_x509_aki_deinit(aki);
- 	return ret;
+	gnutls_x509_aki_deinit(aki);
+	return ret;
 }
