@@ -17,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -38,8 +38,9 @@ static int _gnutls_x509_read_dsa_pubkey(uint8_t * der, int dersize,
 					gnutls_pk_params_st * params);
 static int _gnutls_x509_read_ecc_pubkey(uint8_t * der, int dersize,
 					gnutls_pk_params_st * params);
-static int _gnutls_x509_read_eddsa_pubkey(uint8_t * der, int dersize,
-					gnutls_pk_params_st * params);
+static int _gnutls_x509_read_eddsa_pubkey(gnutls_ecc_curve_t curve,
+					  uint8_t * der, int dersize,
+					  gnutls_pk_params_st * params);
 static int _gnutls_x509_read_gost_pubkey(uint8_t * der, int dersize,
 					gnutls_pk_params_st * params);
 
@@ -57,7 +58,7 @@ _gnutls_x509_read_rsa_pubkey(uint8_t * der, int dersize,
 			     gnutls_pk_params_st * params)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	if ((result = asn1_create_element
 	     (_gnutls_get_gnutls_asn(), "GNUTLS.RSAPublicKey", &spk))
@@ -75,16 +76,15 @@ _gnutls_x509_read_rsa_pubkey(uint8_t * der, int dersize,
 	}
 
 
-	if ((result =
-	     _gnutls_x509_read_int(spk, "modulus",
-				   &params->params[0])) < 0) {
+	if (_gnutls_x509_read_int(spk, "modulus",
+				  &params->params[0]) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(&spk);
 		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
-	if ((result = _gnutls_x509_read_int(spk, "publicExponent",
-					    &params->params[1])) < 0) {
+	if (_gnutls_x509_read_int(spk, "publicExponent",
+				  &params->params[1]) < 0) {
 		gnutls_assert();
 		_gnutls_mpi_release(&params->params[0]);
 		asn1_delete_structure(&spk);
@@ -114,9 +114,14 @@ _gnutls_x509_read_ecc_pubkey(uint8_t * der, int dersize,
 					    &params->params[ECC_Y]);
 }
 
-int _gnutls_x509_read_eddsa_pubkey(uint8_t * der, int dersize,
+int _gnutls_x509_read_eddsa_pubkey(gnutls_ecc_curve_t curve,
+				   uint8_t * der, int dersize,
 				   gnutls_pk_params_st * params)
 {
+	int size = gnutls_ecc_curve_get_size(curve);
+	if (dersize != size)
+		return gnutls_assert_val(GNUTLS_E_ILLEGAL_PARAMETER);
+
 	return _gnutls_set_datum(&params->raw_pub, der, dersize);
 }
 
@@ -170,7 +175,7 @@ _gnutls_x509_read_dsa_params(uint8_t * der, int dersize,
 			     gnutls_pk_params_st * params)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	if ((result = asn1_create_element
 	     (_gnutls_get_pkix(), "PKIX1.Dss-Parms",
@@ -187,15 +192,14 @@ _gnutls_x509_read_dsa_params(uint8_t * der, int dersize,
 		return _gnutls_asn2err(result);
 	}
 
-	/* FIXME: If the parameters are not included in the certificate
+	/* If the parameters are not included in the certificate
 	 * then the issuer's parameters should be used. This is not
-	 * done yet.
+	 * implemented, and is not used in practice (along with DSA).
 	 */
 
 	/* Read p */
 
-	if ((result =
-	     _gnutls_x509_read_int(spk, "p", &params->params[0])) < 0) {
+	if (_gnutls_x509_read_int(spk, "p", &params->params[0]) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(&spk);
 		return GNUTLS_E_ASN1_GENERIC_ERROR;
@@ -203,8 +207,7 @@ _gnutls_x509_read_dsa_params(uint8_t * der, int dersize,
 
 	/* Read q */
 
-	if ((result =
-	     _gnutls_x509_read_int(spk, "q", &params->params[1])) < 0) {
+	if (_gnutls_x509_read_int(spk, "q", &params->params[1]) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(&spk);
 		_gnutls_mpi_release(&params->params[0]);
@@ -213,8 +216,7 @@ _gnutls_x509_read_dsa_params(uint8_t * der, int dersize,
 
 	/* Read g */
 
-	if ((result =
-	     _gnutls_x509_read_int(spk, "g", &params->params[2])) < 0) {
+	if (_gnutls_x509_read_int(spk, "g", &params->params[2]) < 0) {
 		gnutls_assert();
 		asn1_delete_structure(&spk);
 		_gnutls_mpi_release(&params->params[0]);
@@ -239,7 +241,7 @@ _gnutls_x509_read_ecc_params(uint8_t * der, int dersize,
 			     unsigned int * curve)
 {
 	int ret;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 	char oid[MAX_OID_SIZE];
 	int oid_size;
 
@@ -292,8 +294,8 @@ _gnutls_x509_read_rsa_pss_params(uint8_t * der, int dersize,
 				 gnutls_x509_spki_st * params)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
+	asn1_node c2 = NULL;
 	gnutls_digest_algorithm_t digest;
 	char oid[MAX_OID_SIZE] = "";
 	int size;
@@ -440,13 +442,15 @@ _gnutls_x509_read_gost_params(uint8_t * der, int dersize,
 			      gnutls_pk_algorithm_t algo)
 {
 	int ret;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 	char oid[MAX_OID_SIZE];
 	int oid_size;
 	gnutls_ecc_curve_t curve;
 	gnutls_gost_paramset_t param;
 
 	if ((ret = asn1_create_element(_gnutls_get_gnutls_asn(),
+				       algo == GNUTLS_PK_GOST_01 ?
+				       "GNUTLS.GOSTParametersOld" :
 				       "GNUTLS.GOSTParameters",
 				       &spk)) != ASN1_SUCCESS) {
 		gnutls_assert();
@@ -481,7 +485,8 @@ _gnutls_x509_read_gost_params(uint8_t * der, int dersize,
 	/* Read the digest */
 	oid_size = sizeof(oid);
 	ret = asn1_read_value(spk, "digestParamSet", oid, &oid_size);
-	if (ret != ASN1_SUCCESS) {
+	if (ret != ASN1_SUCCESS &&
+	    ret != ASN1_ELEMENT_NOT_FOUND) {
 		gnutls_assert();
 		ret = _gnutls_asn2err(ret);
 		goto cleanup;
@@ -554,7 +559,10 @@ int _gnutls_x509_read_pubkey(gnutls_pk_algorithm_t algo, uint8_t * der,
 		}
 		break;
 	case GNUTLS_PK_EDDSA_ED25519:
-		ret = _gnutls_x509_read_eddsa_pubkey(der, dersize, params);
+		ret = _gnutls_x509_read_eddsa_pubkey(GNUTLS_ECC_CURVE_ED25519, der, dersize, params);
+		break;
+	case GNUTLS_PK_EDDSA_ED448:
+		ret = _gnutls_x509_read_eddsa_pubkey(GNUTLS_ECC_CURVE_ED448, der, dersize, params);
 		break;
 	case GNUTLS_PK_GOST_01:
 	case GNUTLS_PK_GOST_12_256:
@@ -581,6 +589,7 @@ int _gnutls_x509_read_pubkey_params(gnutls_pk_algorithm_t algo,
 	switch (algo) {
 	case GNUTLS_PK_RSA:
 	case GNUTLS_PK_EDDSA_ED25519:
+	case GNUTLS_PK_EDDSA_ED448:
 		return 0;
 	case GNUTLS_PK_RSA_PSS:
 		return _gnutls_x509_read_rsa_pss_params(der, dersize, &params->spki);
@@ -625,6 +634,7 @@ int _gnutls_x509_check_pubkey_params(gnutls_pk_params_st * params)
 	case GNUTLS_PK_DSA:
 	case GNUTLS_PK_ECDSA:
 	case GNUTLS_PK_EDDSA_ED25519:
+	case GNUTLS_PK_EDDSA_ED448:
 	case GNUTLS_PK_GOST_01:
 	case GNUTLS_PK_GOST_12_256:
 	case GNUTLS_PK_GOST_12_512:
