@@ -33,15 +33,15 @@
 #include <mpi.h>
 #include <ecc.h>
 
-static int _gnutls_x509_write_rsa_pubkey(gnutls_pk_params_st * params,
+static int _gnutls_x509_write_rsa_pubkey(const gnutls_pk_params_st * params,
 					 gnutls_datum_t * der);
-static int _gnutls_x509_write_dsa_params(gnutls_pk_params_st * params,
+static int _gnutls_x509_write_dsa_params(const gnutls_pk_params_st * params,
 					 gnutls_datum_t * der);
-static int _gnutls_x509_write_dsa_pubkey(gnutls_pk_params_st * params,
+static int _gnutls_x509_write_dsa_pubkey(const gnutls_pk_params_st * params,
 					 gnutls_datum_t * der);
-static int _gnutls_x509_write_gost_params(gnutls_pk_params_st * params,
+static int _gnutls_x509_write_gost_params(const gnutls_pk_params_st * params,
 					 gnutls_datum_t * der);
-static int _gnutls_x509_write_gost_pubkey(gnutls_pk_params_st * params,
+static int _gnutls_x509_write_gost_pubkey(const gnutls_pk_params_st * params,
 					 gnutls_datum_t * der);
 
 /*
@@ -52,11 +52,11 @@ static int _gnutls_x509_write_gost_pubkey(gnutls_pk_params_st * params,
  * Allocates the space used to store the DER data.
  */
 static int
-_gnutls_x509_write_rsa_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_rsa_pubkey(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	der->data = NULL;
 	der->size = 0;
@@ -110,7 +110,7 @@ _gnutls_x509_write_rsa_pubkey(gnutls_pk_params_st * params,
  * Allocates the space used to store the DER data.
  */
 int
-_gnutls_x509_write_ecc_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_ecc_pubkey(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	int result;
@@ -139,7 +139,7 @@ _gnutls_x509_write_ecc_pubkey(gnutls_pk_params_st * params,
  * Allocates the space used to store the data.
  */
 int
-_gnutls_x509_write_eddsa_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_eddsa_pubkey(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * raw)
 {
 	int ret;
@@ -150,7 +150,8 @@ _gnutls_x509_write_eddsa_pubkey(gnutls_pk_params_st * params,
 	if (params->raw_pub.size == 0)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	if (params->curve != GNUTLS_ECC_CURVE_ED25519)
+	if (params->curve != GNUTLS_ECC_CURVE_ED25519 &&
+	    params->curve != GNUTLS_ECC_CURVE_ED448)
 		return gnutls_assert_val(GNUTLS_E_ECC_UNSUPPORTED_CURVE);
 
 	ret = _gnutls_set_datum(raw, params->raw_pub.data, params->raw_pub.size);
@@ -161,7 +162,7 @@ _gnutls_x509_write_eddsa_pubkey(gnutls_pk_params_st * params,
 }
 
 int
-_gnutls_x509_write_gost_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_gost_pubkey(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	bigint_t x, y;
@@ -233,7 +234,7 @@ _gnutls_x509_write_gost_pubkey(gnutls_pk_params_st * params,
 }
 
 int
-_gnutls_x509_write_pubkey_params(gnutls_pk_params_st * params,
+_gnutls_x509_write_pubkey_params(const gnutls_pk_params_st * params,
 				 gnutls_datum_t * der)
 {
 	switch (params->algo) {
@@ -252,6 +253,7 @@ _gnutls_x509_write_pubkey_params(gnutls_pk_params_st * params,
 	case GNUTLS_PK_ECDSA:
 		return _gnutls_x509_write_ecc_params(params->curve, der);
 	case GNUTLS_PK_EDDSA_ED25519:
+	case GNUTLS_PK_EDDSA_ED448:
 		der->data = NULL;
 		der->size = 0;
 
@@ -266,7 +268,7 @@ _gnutls_x509_write_pubkey_params(gnutls_pk_params_st * params,
 }
 
 int
-_gnutls_x509_write_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_pubkey(const gnutls_pk_params_st * params,
 			  gnutls_datum_t * der)
 {
 	switch (params->algo) {
@@ -278,6 +280,7 @@ _gnutls_x509_write_pubkey(gnutls_pk_params_st * params,
 	case GNUTLS_PK_ECDSA:
 		return _gnutls_x509_write_ecc_pubkey(params, der);
 	case GNUTLS_PK_EDDSA_ED25519:
+	case GNUTLS_PK_EDDSA_ED448:
 		return _gnutls_x509_write_eddsa_pubkey(params, der);
 	case GNUTLS_PK_GOST_01:
 	case GNUTLS_PK_GOST_12_256:
@@ -295,11 +298,11 @@ _gnutls_x509_write_pubkey(gnutls_pk_params_st * params,
  * Allocates the space used to store the DER data.
  */
 static int
-_gnutls_x509_write_dsa_params(gnutls_pk_params_st * params,
+_gnutls_x509_write_dsa_params(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	der->data = NULL;
 	der->size = 0;
@@ -355,11 +358,11 @@ _gnutls_x509_write_dsa_params(gnutls_pk_params_st * params,
  * Allocates the space used to store the DER data.
  */
 int
-_gnutls_x509_write_ecc_params(gnutls_ecc_curve_t curve,
+_gnutls_x509_write_ecc_params(const gnutls_ecc_curve_t curve,
 			      gnutls_datum_t * der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 	const char *oid;
 
 	der->data = NULL;
@@ -406,12 +409,12 @@ _gnutls_x509_write_ecc_params(gnutls_ecc_curve_t curve,
 }
 
 int
-_gnutls_x509_write_rsa_pss_params(gnutls_x509_spki_st *params,
+_gnutls_x509_write_rsa_pss_params(const gnutls_x509_spki_st *params,
 				  gnutls_datum_t *der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
+	asn1_node c2 = NULL;
 	const char *oid;
 	gnutls_datum_t tmp = { NULL, 0 };
 
@@ -524,11 +527,11 @@ _gnutls_x509_write_rsa_pss_params(gnutls_x509_spki_st *params,
 }
 
 static int
-_gnutls_x509_write_gost_params(gnutls_pk_params_st * params,
+_gnutls_x509_write_gost_params(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 	const char *oid;
 
 	der->data = NULL;
@@ -559,7 +562,7 @@ _gnutls_x509_write_gost_params(gnutls_pk_params_st * params,
 
 	/* For compatibility per R 1323565.1.023—2018 provide digest OID only
 	 * for GOST-2001 keys or GOST-2012 keys with CryptoPro curves. Do not
-	 * set this optional paramter for TC26 curves */
+	 * set this optional parameter for TC26 curves */
 	if (params->algo == GNUTLS_PK_GOST_01)
 		oid = HASH_OID_GOST_R_3411_94_CRYPTOPRO_PARAMS;
 	else if (params->algo == GNUTLS_PK_GOST_12_256 &&
@@ -569,6 +572,10 @@ _gnutls_x509_write_gost_params(gnutls_pk_params_st * params,
 		  params->curve == GNUTLS_ECC_CURVE_GOST256CPXA ||
 		  params->curve == GNUTLS_ECC_CURVE_GOST256CPXB))
 		oid = HASH_OID_STREEBOG_256;
+	else if (params->algo == GNUTLS_PK_GOST_12_512 &&
+		 (params->curve == GNUTLS_ECC_CURVE_GOST512A ||
+		  params->curve == GNUTLS_ECC_CURVE_GOST512B))
+		oid = HASH_OID_STREEBOG_512;
 	else
 		oid = NULL;
 
@@ -618,11 +625,11 @@ _gnutls_x509_write_gost_params(gnutls_pk_params_st * params,
  * Allocates the space used to store the DER data.
  */
 static int
-_gnutls_x509_write_dsa_pubkey(gnutls_pk_params_st * params,
+_gnutls_x509_write_dsa_pubkey(const gnutls_pk_params_st * params,
 			      gnutls_datum_t * der)
 {
 	int result;
-	ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+	asn1_node spk = NULL;
 
 	der->data = NULL;
 	der->size = 0;
@@ -662,7 +669,7 @@ _gnutls_x509_write_dsa_pubkey(gnutls_pk_params_st * params,
 /* Encodes the RSA parameters into an ASN.1 RSA private key structure.
  */
 static int
-_gnutls_asn1_encode_rsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
+_gnutls_asn1_encode_rsa(asn1_node * c2, gnutls_pk_params_st * params)
 {
 	int result, ret;
 	uint8_t null = '\0';
@@ -689,9 +696,9 @@ _gnutls_asn1_encode_rsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 	 */
 
 	/* first make sure that no previously allocated data are leaked */
-	if (*c2 != ASN1_TYPE_EMPTY) {
+	if (*c2 != NULL) {
 		asn1_delete_structure(c2);
-		*c2 = ASN1_TYPE_EMPTY;
+		*c2 = NULL;
 	}
 
 	if ((result = asn1_create_element
@@ -702,7 +709,7 @@ _gnutls_asn1_encode_rsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 		goto cleanup;
 	}
 
-	/* Write PRIME 
+	/* Write PRIME
 	 */
 	ret =
 	    _gnutls_x509_write_int(*c2, "modulus",
@@ -796,7 +803,7 @@ _gnutls_asn1_encode_rsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 /* Encodes the ECC parameters into an ASN.1 ECPrivateKey structure.
  */
 static int
-_gnutls_asn1_encode_ecc(ASN1_TYPE * c2, gnutls_pk_params_st * params)
+_gnutls_asn1_encode_ecc(asn1_node * c2, gnutls_pk_params_st * params)
 {
 	int ret;
 	uint8_t one = '\x01';
@@ -808,9 +815,9 @@ _gnutls_asn1_encode_ecc(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 	/* first make sure that no previously allocated data are leaked */
-	if (*c2 != ASN1_TYPE_EMPTY) {
+	if (*c2 != NULL) {
 		asn1_delete_structure(c2);
-		*c2 = ASN1_TYPE_EMPTY;
+		*c2 = NULL;
 	}
 
 	if ((ret = asn1_create_element
@@ -902,7 +909,7 @@ cleanup:
 }
 
 static int
-_gnutls_asn1_encode_gost(ASN1_TYPE * c2, gnutls_pk_params_st * params)
+_gnutls_asn1_encode_gost(asn1_node * c2, gnutls_pk_params_st * params)
 {
 	int ret;
 	const char *oid;
@@ -913,9 +920,9 @@ _gnutls_asn1_encode_gost(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 	/* first make sure that no previously allocated data are leaked */
-	if (*c2 != ASN1_TYPE_EMPTY) {
+	if (*c2 != NULL) {
 		asn1_delete_structure(c2);
-		*c2 = ASN1_TYPE_EMPTY;
+		*c2 = NULL;
 	}
 
 	if ((ret = asn1_create_element
@@ -945,15 +952,15 @@ cleanup:
 /* Encodes the DSA parameters into an ASN.1 DSAPrivateKey structure.
  */
 static int
-_gnutls_asn1_encode_dsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
+_gnutls_asn1_encode_dsa(asn1_node * c2, gnutls_pk_params_st * params)
 {
 	int result, ret;
 	const uint8_t null = '\0';
 
 	/* first make sure that no previously allocated data are leaked */
-	if (*c2 != ASN1_TYPE_EMPTY) {
+	if (*c2 != NULL) {
 		asn1_delete_structure(c2);
-		*c2 = ASN1_TYPE_EMPTY;
+		*c2 = NULL;
 	}
 
 	if ((result = asn1_create_element
@@ -963,7 +970,7 @@ _gnutls_asn1_encode_dsa(ASN1_TYPE * c2, gnutls_pk_params_st * params)
 		return _gnutls_asn2err(result);
 	}
 
-	/* Write PRIME 
+	/* Write PRIME
 	 */
 	ret =
 	    _gnutls_x509_write_int(*c2, "p",
@@ -1020,7 +1027,7 @@ cleanup:
 	return ret;
 }
 
-int _gnutls_asn1_encode_privkey(ASN1_TYPE * c2,
+int _gnutls_asn1_encode_privkey(asn1_node * c2,
 				gnutls_pk_params_st * params)
 {
 	switch (params->algo) {
@@ -1031,6 +1038,7 @@ int _gnutls_asn1_encode_privkey(ASN1_TYPE * c2,
 		return _gnutls_asn1_encode_dsa(c2, params);
 	case GNUTLS_PK_ECDSA:
 	case GNUTLS_PK_EDDSA_ED25519:
+	case GNUTLS_PK_EDDSA_ED448:
 		return _gnutls_asn1_encode_ecc(c2, params);
 	case GNUTLS_PK_GOST_01:
 	case GNUTLS_PK_GOST_12_256:
