@@ -38,16 +38,17 @@
 #include <auth/cert.h>
 
 #include <assert.h>
+#include "intprops.h"
 
 typedef struct gnutls_ocsp_req_int {
-	ASN1_TYPE req;
+	asn1_node req;
 	unsigned init;
 } gnutls_ocsp_req_int;
 
 typedef struct gnutls_ocsp_resp_int {
-	ASN1_TYPE resp;
+	asn1_node resp;
 	gnutls_datum_t response_type_oid;
-	ASN1_TYPE basicresp;
+	asn1_node basicresp;
 	gnutls_datum_t der;
 	unsigned init;
 } gnutls_ocsp_resp_int;
@@ -324,7 +325,7 @@ gnutls_ocsp_resp_import2(gnutls_ocsp_resp_t resp,
 	}
 #define OCSP_BASIC "1.3.6.1.5.5.7.48.1.1"
 
-	if (resp->response_type_oid.size == sizeof(OCSP_BASIC)
+	if (resp->response_type_oid.size == sizeof(OCSP_BASIC) - 1
 	    && memcmp(resp->response_type_oid.data, OCSP_BASIC,
 		      resp->response_type_oid.size) == 0) {
 
@@ -366,7 +367,7 @@ cleanup:
  * Returns: In case of failure a negative error code will be
  *   returned, and 0 on success.
  **/
-int gnutls_ocsp_req_export(gnutls_ocsp_req_t req, gnutls_datum_t * data)
+int gnutls_ocsp_req_export(gnutls_ocsp_req_const_t req, gnutls_datum_t * data)
 {
 	int ret;
 
@@ -398,7 +399,7 @@ int gnutls_ocsp_req_export(gnutls_ocsp_req_t req, gnutls_datum_t * data)
  * Returns: In case of failure a negative error code will be
  *   returned, and 0 on success.
  **/
-int gnutls_ocsp_resp_export(gnutls_ocsp_resp_t resp, gnutls_datum_t * data)
+int gnutls_ocsp_resp_export(gnutls_ocsp_resp_const_t resp, gnutls_datum_t * data)
 {
 	return gnutls_ocsp_resp_export2(resp, data, GNUTLS_X509_FMT_DER);
 }
@@ -416,7 +417,7 @@ int gnutls_ocsp_resp_export(gnutls_ocsp_resp_t resp, gnutls_datum_t * data)
  *
  * Since: 3.6.3
  **/
-int gnutls_ocsp_resp_export2(gnutls_ocsp_resp_t resp, gnutls_datum_t * data,
+int gnutls_ocsp_resp_export2(gnutls_ocsp_resp_const_t resp, gnutls_datum_t * data,
 			     gnutls_x509_crt_fmt_t fmt)
 {
 	int ret;
@@ -454,27 +455,14 @@ int gnutls_ocsp_resp_export2(gnutls_ocsp_resp_t resp, gnutls_datum_t * data,
  *
  * Returns: version of OCSP request, or a negative error code on error.
  **/
-int gnutls_ocsp_req_get_version(gnutls_ocsp_req_t req)
+int gnutls_ocsp_req_get_version(gnutls_ocsp_req_const_t req)
 {
-	uint8_t version[8];
-	int len, ret;
-
 	if (req == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	len = sizeof(version);
-	ret =
-	    asn1_read_value(req->req, "tbsRequest.version", version, &len);
-	if (ret != ASN1_SUCCESS) {
-		if (ret == ASN1_ELEMENT_NOT_FOUND)
-			return 1;	/* the DEFAULT version */
-		gnutls_assert();
-		return _gnutls_asn2err(ret);
-	}
-
-	return (int) version[0] + 1;
+	return _gnutls_x509_get_version(req->req, "tbsRequest.version");
 }
 
 /**
@@ -507,7 +495,7 @@ int gnutls_ocsp_req_get_version(gnutls_ocsp_req_t req)
  *   returned.
  **/
 int
-gnutls_ocsp_req_get_cert_id(gnutls_ocsp_req_t req,
+gnutls_ocsp_req_get_cert_id(gnutls_ocsp_req_const_t req,
 			    unsigned indx,
 			    gnutls_digest_algorithm_t * digest,
 			    gnutls_datum_t * issuer_name_hash,
@@ -808,13 +796,15 @@ gnutls_ocsp_req_add_cert(gnutls_ocsp_req_t req,
  * The caller needs to deallocate memory by calling gnutls_free() on
  * @oid->data and @data->data.
  *
+ * Since 3.7.0 @oid->size does not account for the terminating null byte.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error code is returned.  If you have reached the last
  *   extension available %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE will
  *   be returned.
  **/
 int
-gnutls_ocsp_req_get_extension(gnutls_ocsp_req_t req,
+gnutls_ocsp_req_get_extension(gnutls_ocsp_req_const_t req,
 			      unsigned indx,
 			      gnutls_datum_t * oid,
 			      unsigned int *critical,
@@ -919,7 +909,7 @@ gnutls_ocsp_req_set_extension(gnutls_ocsp_req_t req,
  *   negative error code is returned.
  **/
 int
-gnutls_ocsp_req_get_nonce(gnutls_ocsp_req_t req,
+gnutls_ocsp_req_get_nonce(gnutls_ocsp_req_const_t req,
 			  unsigned int *critical, gnutls_datum_t * nonce)
 {
 	int ret;
@@ -1049,7 +1039,7 @@ int gnutls_ocsp_req_randomize_nonce(gnutls_ocsp_req_t req)
  * Returns: status of OCSP request as a #gnutls_ocsp_resp_status_t, or
  *   a negative error code on error.
  **/
-int gnutls_ocsp_resp_get_status(gnutls_ocsp_resp_t resp)
+int gnutls_ocsp_resp_get_status(gnutls_ocsp_resp_const_t resp)
 {
 	uint8_t str[1];
 	int len, ret;
@@ -1101,11 +1091,14 @@ int gnutls_ocsp_resp_get_status(gnutls_ocsp_resp_t resp)
  * Otherwise gnutls_ocsp_resp_import() will decode the basic OCSP
  * response part and the caller need not worry about that aspect.
  *
+ * Since 3.7.0 @response_type_oid->size does not account for the terminating
+ * null byte.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_get_response(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_response(gnutls_ocsp_resp_const_t resp,
 			      gnutls_datum_t * response_type_oid,
 			      gnutls_datum_t * response)
 {
@@ -1151,28 +1144,14 @@ gnutls_ocsp_resp_get_response(gnutls_ocsp_resp_t resp,
  * Returns: version of Basic OCSP response, or a negative error code
  *   on error.
  **/
-int gnutls_ocsp_resp_get_version(gnutls_ocsp_resp_t resp)
+int gnutls_ocsp_resp_get_version(gnutls_ocsp_resp_const_t resp)
 {
-	uint8_t version[8];
-	int len, ret;
-
 	if (resp == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	len = sizeof(version);
-	ret =
-	    asn1_read_value(resp->resp, "tbsResponseData.version", version,
-			    &len);
-	if (ret != ASN1_SUCCESS) {
-		if (ret == ASN1_ELEMENT_NOT_FOUND)
-			return 1;	/* the DEFAULT version */
-		gnutls_assert();
-		return _gnutls_asn2err(ret);
-	}
-
-	return (int) version[0] + 1;
+	return _gnutls_x509_get_version(resp->resp, "tbsResponseData.version");
 }
 
 /**
@@ -1199,7 +1178,7 @@ int gnutls_ocsp_resp_get_version(gnutls_ocsp_resp_t resp)
  *   return success and set @dn elements to zero.
  **/
 int
-gnutls_ocsp_resp_get_responder(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_responder(gnutls_ocsp_resp_const_t resp,
 			       gnutls_datum_t * dn)
 {
 	int ret;
@@ -1240,7 +1219,7 @@ gnutls_ocsp_resp_get_responder(gnutls_ocsp_resp_t resp,
  *   %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE.
  **/
 int
-gnutls_ocsp_resp_get_responder2(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_responder2(gnutls_ocsp_resp_const_t resp,
 			        gnutls_datum_t * dn, unsigned flags)
 {
 	if (resp == NULL || dn == NULL) {
@@ -1273,7 +1252,7 @@ gnutls_ocsp_resp_get_responder2(gnutls_ocsp_resp_t resp,
  *   negative error code is returned.
  **/
 int
-gnutls_ocsp_resp_get_responder_raw_id(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_responder_raw_id(gnutls_ocsp_resp_const_t resp,
 				      unsigned type,
 				      gnutls_datum_t * raw)
 {
@@ -1341,7 +1320,7 @@ gnutls_ocsp_resp_get_responder_raw_id(gnutls_ocsp_resp_t resp,
  *
  * Returns: signing time, or (time_t)-1 on error.
  **/
-time_t gnutls_ocsp_resp_get_produced(gnutls_ocsp_resp_t resp)
+time_t gnutls_ocsp_resp_get_produced(gnutls_ocsp_resp_const_t resp)
 {
 	char ttime[MAX_TIME];
 	int len, ret;
@@ -1381,7 +1360,7 @@ time_t gnutls_ocsp_resp_get_produced(gnutls_ocsp_resp_t resp)
  * Since: 3.1.3
  **/
 int
-gnutls_ocsp_resp_check_crt(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_check_crt(gnutls_ocsp_resp_const_t resp,
 			   unsigned int indx, gnutls_x509_crt_t crt)
 {
 	int ret;
@@ -1493,7 +1472,7 @@ gnutls_ocsp_resp_check_crt(gnutls_ocsp_resp_t resp,
  *   returned.
  **/
 int
-gnutls_ocsp_resp_get_single(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_single(gnutls_ocsp_resp_const_t resp,
 			    unsigned indx,
 			    gnutls_digest_algorithm_t * digest,
 			    gnutls_datum_t * issuer_name_hash,
@@ -1696,13 +1675,15 @@ gnutls_ocsp_resp_get_single(gnutls_ocsp_resp_t resp,
  * The caller needs to deallocate memory by calling gnutls_free() on
  * @oid->data and @data->data.
  *
+ * Since 3.7.0 @oid->size does not account for the terminating null byte.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error code is returned.  If you have reached the last
  *   extension available %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE will
  *   be returned.
  **/
 int
-gnutls_ocsp_resp_get_extension(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_extension(gnutls_ocsp_resp_const_t resp,
 			       unsigned indx,
 			       gnutls_datum_t * oid,
 			       unsigned int *critical,
@@ -1780,7 +1761,7 @@ gnutls_ocsp_resp_get_extension(gnutls_ocsp_resp_t resp,
  *   negative error code is returned.
  **/
 int
-gnutls_ocsp_resp_get_nonce(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_nonce(gnutls_ocsp_resp_const_t resp,
 			   unsigned int *critical, gnutls_datum_t * nonce)
 {
 	int ret;
@@ -1820,7 +1801,7 @@ gnutls_ocsp_resp_get_nonce(gnutls_ocsp_resp_t resp,
  * Returns: a #gnutls_sign_algorithm_t value, or a negative error code
  *   on error.
  **/
-int gnutls_ocsp_resp_get_signature_algorithm(gnutls_ocsp_resp_t resp)
+int gnutls_ocsp_resp_get_signature_algorithm(gnutls_ocsp_resp_const_t resp)
 {
 	int ret;
 	gnutls_datum_t sa;
@@ -1850,7 +1831,7 @@ int gnutls_ocsp_resp_get_signature_algorithm(gnutls_ocsp_resp_t resp)
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_get_signature(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_signature(gnutls_ocsp_resp_const_t resp,
 			       gnutls_datum_t * sig)
 {
 	int ret;
@@ -1892,7 +1873,7 @@ gnutls_ocsp_resp_get_signature(gnutls_ocsp_resp_t resp,
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_get_certs(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_get_certs(gnutls_ocsp_resp_const_t resp,
 			   gnutls_x509_crt_t ** certs, size_t * ncerts)
 {
 	int ret;
@@ -1925,9 +1906,13 @@ gnutls_ocsp_resp_get_certs(gnutls_ocsp_resp_t resp,
 			goto error;
 		}
 
-		tmpcerts2 =
-		    gnutls_realloc_fast(tmpcerts,
-					(ctr + 2) * sizeof(*tmpcerts));
+		if (unlikely(INT_ADD_OVERFLOW(ctr, 2))) {
+			ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+			goto error;
+		}
+
+		tmpcerts2 = _gnutls_reallocarray_fast(tmpcerts, ctr + 2,
+						      sizeof(*tmpcerts));
 		if (tmpcerts2 == NULL) {
 			gnutls_assert();
 			ret = GNUTLS_E_MEMORY_ERROR;
@@ -1976,7 +1961,7 @@ gnutls_ocsp_resp_get_certs(gnutls_ocsp_resp_t resp,
 
 /* Search the OCSP response for a certificate matching the responderId
    mentioned in the OCSP response. */
-static gnutls_x509_crt_t find_signercert(gnutls_ocsp_resp_t resp)
+static gnutls_x509_crt_t find_signercert(gnutls_ocsp_resp_const_t resp)
 {
 	int rc;
 	gnutls_x509_crt_t *certs = NULL;
@@ -2096,7 +2081,7 @@ static gnutls_x509_crt_t find_signercert(gnutls_ocsp_resp_t resp)
 }
 
 static int
-_ocsp_resp_verify_direct(gnutls_ocsp_resp_t resp,
+_ocsp_resp_verify_direct(gnutls_ocsp_resp_const_t resp,
 			 gnutls_x509_crt_t signercert,
 			 unsigned int *verify, unsigned int flags)
 {
@@ -2130,7 +2115,7 @@ _ocsp_resp_verify_direct(gnutls_ocsp_resp_t resp,
 		goto done;
 	}
 
-	_gnutls_cert_log("ocsp signer", signercert); \
+	_gnutls_cert_log("ocsp signer", signercert);
 
 	rc = gnutls_pubkey_import_x509(pubkey, signercert, 0);
 	if (rc != GNUTLS_E_SUCCESS) {
@@ -2236,7 +2221,7 @@ static int check_ocsp_purpose(gnutls_x509_crt_t signercert)
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_verify_direct(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_verify_direct(gnutls_ocsp_resp_const_t resp,
 			       gnutls_x509_crt_t issuer,
 			       unsigned int *verify, unsigned int flags)
 {
@@ -2321,7 +2306,7 @@ gnutls_ocsp_resp_verify_direct(gnutls_ocsp_resp_t resp,
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_verify(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_verify(gnutls_ocsp_resp_const_t resp,
 			gnutls_x509_trust_list_t trustlist,
 			unsigned int *verify, unsigned int flags)
 {
@@ -2478,7 +2463,14 @@ gnutls_ocsp_resp_list_import2(gnutls_ocsp_resp_t **ocsps,
 				goto fail;
 			}
 
-			new_ocsps = gnutls_realloc(*ocsps, (*size + 1)*sizeof(gnutls_ocsp_resp_t));
+			if (unlikely(INT_ADD_OVERFLOW(*size, 1))) {
+				ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+				goto fail;
+			}
+
+			new_ocsps = _gnutls_reallocarray(*ocsps,
+							 *size + 1,
+							 sizeof(gnutls_ocsp_resp_t));
 			if (new_ocsps == NULL) {
 				resp = NULL;
 				gnutls_assert();
@@ -2512,7 +2504,7 @@ gnutls_ocsp_resp_list_import2(gnutls_ocsp_resp_t **ocsps,
 			goto cleanup;
 		}
 
-		*ocsps = gnutls_malloc(1*sizeof(gnutls_ocsp_resp_t));
+		*ocsps = gnutls_malloc(sizeof(gnutls_ocsp_resp_t));
 		if (*ocsps == NULL) {
 			gnutls_assert();
 			ret = GNUTLS_E_MEMORY_ERROR;
@@ -2544,7 +2536,7 @@ gnutls_ocsp_resp_list_import2(gnutls_ocsp_resp_t **ocsps,
  * time, and would otherwise treat it as too old.
  * Otherwise it returns the time after which that data  is invalid.
  */
-time_t _gnutls_ocsp_get_validity(gnutls_ocsp_resp_t resp)
+time_t _gnutls_ocsp_get_validity(gnutls_ocsp_resp_const_t resp)
 {
 	unsigned int cert_status;
 	time_t rtime, vtime, ntime, now;
