@@ -31,6 +31,7 @@
 #include "x509_ext_int.h"
 #include "virt-san.h"
 #include <gnutls/x509-ext.h>
+#include "intprops.h"
 
 #define MAX_ENTRIES 64
 struct gnutls_subject_alt_names_st {
@@ -137,7 +138,11 @@ int subject_alt_names_set(struct name_st **names,
 	void *tmp;
 	int ret;
 
-	tmp = gnutls_realloc(*names, (*size + 1) * sizeof((*names)[0]));
+	if (unlikely(INT_ADD_OVERFLOW(*size, 1))) {
+		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+	}
+
+	tmp = _gnutls_reallocarray(*names, *size + 1, sizeof((*names)[0]));
 	if (tmp == NULL) {
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
@@ -216,7 +221,7 @@ int gnutls_x509_ext_import_subject_alt_names(const gnutls_datum_t * ext,
 					  gnutls_subject_alt_names_t sans,
 					  unsigned int flags)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result, ret;
 	unsigned int i;
 	gnutls_datum_t san, othername_oid;
@@ -236,8 +241,7 @@ int gnutls_x509_ext_import_subject_alt_names(const gnutls_datum_t * ext,
 		goto cleanup;
 	}
 
-	i = 0;
-	do {
+	for (i=0;;i++) {
 		san.data = NULL;
 		san.size = 0;
 		othername_oid.data = NULL;
@@ -264,9 +268,7 @@ int gnutls_x509_ext_import_subject_alt_names(const gnutls_datum_t * ext,
 					    (char *)othername_oid.data, 1);
 		if (ret < 0)
 			break;
-
-		i++;
-	} while (ret >= 0);
+	}
 
 	sans->size = i;
 	if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
@@ -298,7 +300,7 @@ int gnutls_x509_ext_import_subject_alt_names(const gnutls_datum_t * ext,
 int gnutls_x509_ext_export_subject_alt_names(gnutls_subject_alt_names_t sans,
 					  gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result, ret;
 	unsigned i;
 
@@ -369,7 +371,7 @@ int gnutls_x509_ext_import_name_constraints(const gnutls_datum_t * ext,
 					 unsigned int flags)
 {
 	int result, ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	gnutls_x509_name_constraints_t nc2 = NULL;
 
 	result = asn1_create_element
@@ -464,7 +466,7 @@ int gnutls_x509_ext_export_name_constraints(gnutls_x509_name_constraints_t nc,
 {
 	int ret, result;
 	uint8_t null = 0;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	struct name_constraints_node_st *tmp;
 
 	if (nc->permitted == NULL && nc->excluded == NULL)
@@ -603,7 +605,7 @@ int gnutls_x509_ext_import_subject_key_id(const gnutls_datum_t * ext,
 				       gnutls_datum_t * id)
 {
 	int result, ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	if (ext->size == 0 || ext->data == NULL) {
 		gnutls_assert();
@@ -655,7 +657,7 @@ int gnutls_x509_ext_import_subject_key_id(const gnutls_datum_t * ext,
 int gnutls_x509_ext_export_subject_key_id(const gnutls_datum_t * id,
 				       gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int ret, result;
 
 	result =
@@ -897,7 +899,7 @@ int gnutls_x509_ext_import_authority_key_id(const gnutls_datum_t * ext,
 {
 	int ret;
 	unsigned i;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	gnutls_datum_t san, othername_oid;
 	unsigned type;
 
@@ -916,8 +918,7 @@ int gnutls_x509_ext_import_authority_key_id(const gnutls_datum_t * ext,
 	}
 
 	/* Read authorityCertIssuer */
-	i = 0;
-	do {
+	for (i=0;;i++) {
 		san.data = NULL;
 		san.size = 0;
 		othername_oid.data = NULL;
@@ -944,9 +945,7 @@ int gnutls_x509_ext_import_authority_key_id(const gnutls_datum_t * ext,
 					    (char *)othername_oid.data, 1);
 		if (ret < 0)
 			break;
-
-		i++;
-	} while (ret >= 0);
+	}
 
 	aki->cert_issuer.size = i;
 	if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE
@@ -1000,7 +999,7 @@ int gnutls_x509_ext_import_authority_key_id(const gnutls_datum_t * ext,
 int gnutls_x509_ext_export_authority_key_id(gnutls_x509_aki_t aki,
 					 gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	unsigned i;
 	int result, ret;
 
@@ -1095,7 +1094,7 @@ int gnutls_x509_ext_export_authority_key_id(gnutls_x509_aki_t aki,
 int gnutls_x509_ext_import_key_usage(const gnutls_datum_t * ext,
 				  unsigned int *key_usage)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int len, result;
 	uint8_t str[2];
 
@@ -1130,6 +1129,33 @@ int gnutls_x509_ext_import_key_usage(const gnutls_datum_t * ext,
 	return 0;
 }
 
+static int _last_key_usage_set_bit(int usage)
+{
+/* the byte ordering is a bit strange here, see how GNUTLS_KEY_* is laid out, and how 
+ * asn1_write_value() writes out BIT STRING objects.
+ */
+        if (usage & GNUTLS_KEY_DECIPHER_ONLY)
+                return 9;
+        else if (usage & GNUTLS_KEY_ENCIPHER_ONLY)
+                return 8;
+        else if (usage & GNUTLS_KEY_CRL_SIGN)
+                return 7;
+        else if (usage & GNUTLS_KEY_KEY_CERT_SIGN)
+                return 6;
+        else if (usage & GNUTLS_KEY_KEY_AGREEMENT)
+                return 5;
+        else if (usage & GNUTLS_KEY_DATA_ENCIPHERMENT)
+                return 4;
+        else if (usage & GNUTLS_KEY_KEY_ENCIPHERMENT)
+                return 3;
+        else if (usage & GNUTLS_KEY_NON_REPUDIATION)
+                return 2;
+        else if (usage & GNUTLS_KEY_DIGITAL_SIGNATURE)
+                return 1;
+        else
+                return 0;
+}
+
 /**
  * gnutls_x509_ext_export_key_usage:
  * @usage: an ORed sequence of the GNUTLS_KEY_* elements.
@@ -1146,7 +1172,7 @@ int gnutls_x509_ext_import_key_usage(const gnutls_datum_t * ext,
  **/
 int gnutls_x509_ext_export_key_usage(unsigned int usage, gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result;
 	uint8_t str[2];
 
@@ -1160,8 +1186,8 @@ int gnutls_x509_ext_export_key_usage(unsigned int usage, gnutls_datum_t * ext)
 	str[1] = usage >> 8;
 
 	/* Since KeyUsage is a BIT STRING, the input to asn1_write_value
-	 * is the number of bits to be read. */
-	result = asn1_write_value(c2, "", str, 9);
+	 * is the number of bits to be written/read. */
+	result = asn1_write_value(c2, "", str, _last_key_usage_set_bit(usage));
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		asn1_delete_structure(&c2);
@@ -1228,7 +1254,7 @@ int gnutls_x509_ext_import_inhibit_anypolicy(const gnutls_datum_t * ext,
  **/
 int gnutls_x509_ext_export_inhibit_anypolicy(unsigned int skipcerts, gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result, ret;
 
 	result = asn1_create_element(_gnutls_get_gnutls_asn(), "GNUTLS.DSAPublicKey", &c2);
@@ -1277,7 +1303,7 @@ int gnutls_x509_ext_import_private_key_usage_period(const gnutls_datum_t * ext,
 						 time_t * expiration)
 {
 	int result, ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	result = asn1_create_element
 	    (_gnutls_get_pkix(), "PKIX1.PrivateKeyUsagePeriod", &c2);
@@ -1330,7 +1356,7 @@ int gnutls_x509_ext_export_private_key_usage_period(time_t activation,
 						 gnutls_datum_t * ext)
 {
 	int result;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	result =
 	    asn1_create_element(_gnutls_get_pkix(),
@@ -1382,7 +1408,7 @@ int gnutls_x509_ext_export_private_key_usage_period(time_t activation,
 int gnutls_x509_ext_import_basic_constraints(const gnutls_datum_t * ext,
 					  unsigned int *ca, int *pathlen)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	char str[128]="";
 	int len, result;
 
@@ -1450,7 +1476,7 @@ int gnutls_x509_ext_import_basic_constraints(const gnutls_datum_t * ext,
 int gnutls_x509_ext_export_basic_constraints(unsigned int ca, int pathlen,
 					  gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	const char *str;
 	int result;
 
@@ -1524,7 +1550,7 @@ int gnutls_x509_ext_import_proxy(const gnutls_datum_t *ext, int *pathlen,
 			         char **policyLanguage, char **policy,
 			         size_t *sizeof_policy)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result;
 	gnutls_datum_t value1 = { NULL, 0 };
 	gnutls_datum_t value2 = { NULL, 0 };
@@ -1617,7 +1643,7 @@ int gnutls_x509_ext_export_proxy(int pathLenConstraint, const char *policyLangua
 			      const char *policy, size_t sizeof_policy,
 			      gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result;
 
 	result = asn1_create_element(_gnutls_get_pkix(),
@@ -1677,7 +1703,7 @@ int gnutls_x509_ext_export_proxy(int pathLenConstraint, const char *policyLangua
 static int decode_user_notice(const void *data, size_t size,
 			      gnutls_datum_t * txt)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int ret, len;
 	char choice_type[64];
 	char name[128];
@@ -1894,7 +1920,7 @@ int gnutls_x509_ext_import_policies(const gnutls_datum_t * ext,
 				 gnutls_x509_policies_t policies,
 				 unsigned int flags)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	char tmpstr[128];
 	char tmpoid[MAX_OID_SIZE];
 	gnutls_datum_t tmpd = { NULL, 0 };
@@ -2035,7 +2061,7 @@ static int encode_user_notice(const gnutls_datum_t * txt,
 			      gnutls_datum_t * der_data)
 {
 	int result;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	if ((result =
 	     asn1_create_element(_gnutls_get_pkix(),
@@ -2103,7 +2129,7 @@ int gnutls_x509_ext_export_policies(gnutls_x509_policies_t policies,
 	int result;
 	unsigned i, j;
 	gnutls_datum_t der_data = {NULL, 0}, tmpd;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	const char *oid;
 
 	result =
@@ -2135,6 +2161,16 @@ int gnutls_x509_ext_export_policies(gnutls_x509_policies_t policies,
 			goto cleanup;
 		}
 
+                if (policies->policy[j].qualifiers == 0) {
+                        /* remove the optional policyQualifiers if none are present. */
+                        result = asn1_write_value(c2, "?LAST.policyQualifiers", NULL, 0); 
+                        if (result != ASN1_SUCCESS) {
+                                gnutls_assert();
+                                result = _gnutls_asn2err(result);
+                                goto cleanup;
+                        }
+                }
+
 		for (i = 0;
 		     i < MIN(policies->policy[j].qualifiers,
 			     GNUTLS_MAX_QUALIFIERS); i++) {
@@ -2322,10 +2358,13 @@ int crl_dist_points_set(gnutls_x509_crl_dist_points_t cdp,
 {
 	void *tmp;
 
+	if (unlikely(INT_ADD_OVERFLOW(cdp->size, 1))) {
+		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+	}
+
 	/* new dist point */
-	tmp =
-	    gnutls_realloc(cdp->points,
-			   (cdp->size + 1) * sizeof(cdp->points[0]));
+	tmp = _gnutls_reallocarray(cdp->points, cdp->size + 1,
+				   sizeof(cdp->points[0]));
 	if (tmp == NULL) {
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
@@ -2394,7 +2433,7 @@ int gnutls_x509_ext_import_crl_dist_points(const gnutls_datum_t * ext,
 					unsigned int flags)
 {
 	int result;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	char name[MAX_NAME_SIZE];
 	int len, ret;
 	uint8_t reasons[2];
@@ -2444,8 +2483,7 @@ int gnutls_x509_ext_import_crl_dist_points(const gnutls_datum_t * ext,
 		snprintf(name, sizeof(name),
 			 "?%u.distributionPoint.fullName", (unsigned)i + 1);
 
-		j = 0;
-		do {
+		for (j=0;;j++) {
 			san.data = NULL;
 			san.size = 0;
 
@@ -2464,9 +2502,7 @@ int gnutls_x509_ext_import_crl_dist_points(const gnutls_datum_t * ext,
 			if (ret < 0)
 				break;
 			san.data = NULL; /* it is now in cdp */
-
-			j++;
-		} while (ret >= 0);
+		}
 
 		i++;
 	} while (ret >= 0);
@@ -2500,7 +2536,7 @@ int gnutls_x509_ext_import_crl_dist_points(const gnutls_datum_t * ext,
 int gnutls_x509_ext_export_crl_dist_points(gnutls_x509_crl_dist_points_t cdp,
 					gnutls_datum_t * ext)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int result;
 	uint8_t reasons[2];
 	unsigned i;
@@ -2743,7 +2779,11 @@ int gnutls_x509_aia_set(gnutls_x509_aia_t aia,
 	void *tmp;
 	unsigned indx;
 
-	tmp = gnutls_realloc(aia->aia, (aia->size + 1) * sizeof(aia->aia[0]));
+	if (unlikely(INT_ADD_OVERFLOW(aia->size, 1))) {
+		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+	}
+
+	tmp = _gnutls_reallocarray(aia->aia, aia->size + 1, sizeof(aia->aia[0]));
 	if (tmp == NULL) {
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
@@ -2769,7 +2809,7 @@ int gnutls_x509_aia_set(gnutls_x509_aia_t aia,
 }
 
 
-static int parse_aia(ASN1_TYPE c2, gnutls_x509_aia_t aia)
+static int parse_aia(asn1_node c2, gnutls_x509_aia_t aia)
 {
 	int len;
 	char nptr[MAX_NAME_SIZE];
@@ -2795,7 +2835,11 @@ static int parse_aia(ASN1_TYPE c2, gnutls_x509_aia_t aia)
 		}
 
 		indx = aia->size;
-		tmp = gnutls_realloc(aia->aia, (aia->size + 1) * sizeof(aia->aia[0]));
+		if (unlikely(INT_ADD_OVERFLOW(aia->size, 1))) {
+			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+		}
+		tmp = _gnutls_reallocarray(aia->aia, aia->size + 1,
+					   sizeof(aia->aia[0]));
 		if (tmp == NULL) {
 			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 		}
@@ -2848,7 +2892,7 @@ int gnutls_x509_ext_import_aia(const gnutls_datum_t * ext,
 					      unsigned int flags)
 {
 	int ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	if (ext->size == 0 || ext->data == NULL) {
 		gnutls_assert();
@@ -2899,7 +2943,7 @@ int gnutls_x509_ext_export_aia(gnutls_x509_aia_t aia,
 					      gnutls_datum_t * ext)
 {
 	int ret, result;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	unsigned int i;
 
 	ret = asn1_create_element(_gnutls_get_pkix(),
@@ -3074,7 +3118,7 @@ int gnutls_x509_ext_import_key_purposes(const gnutls_datum_t * ext,
 {
 	char tmpstr[MAX_NAME_SIZE];
 	int result, ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	gnutls_datum_t oid = {NULL, 0};
 	unsigned i;
 
@@ -3145,7 +3189,7 @@ int gnutls_x509_ext_export_key_purposes(gnutls_x509_key_purposes_t p,
 				     gnutls_datum_t * ext)
 {
 	int result, ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	unsigned i;
 
 	result = asn1_create_element
@@ -3206,7 +3250,7 @@ void gnutls_x509_ext_deinit(gnutls_x509_ext_st *ext)
 
 int _gnutls_x509_decode_ext(const gnutls_datum_t *der, gnutls_x509_ext_st *out)
 {
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	char str_critical[10];
 	char oid[MAX_OID_SIZE];
 	int result, len, ret;
@@ -3275,7 +3319,7 @@ int _gnutls_x509_decode_ext(const gnutls_datum_t *der, gnutls_x509_ext_st *out)
 
 /* flags can be zero or GNUTLS_EXT_FLAG_APPEND
  */
-static int parse_tlsfeatures(ASN1_TYPE c2, gnutls_x509_tlsfeatures_t f, unsigned flags)
+static int parse_tlsfeatures(asn1_node c2, gnutls_x509_tlsfeatures_t f, unsigned flags)
 {
 	char nptr[MAX_NAME_SIZE];
 	int result;
@@ -3351,7 +3395,7 @@ int gnutls_x509_ext_import_tlsfeatures(const gnutls_datum_t * ext,
 				       unsigned int flags)
 {
 	int ret;
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 
 	if (ext->size == 0 || ext->data == NULL) {
 		gnutls_assert();
@@ -3404,7 +3448,7 @@ int gnutls_x509_ext_export_tlsfeatures(gnutls_x509_tlsfeatures_t f,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	asn1_node c2 = NULL;
 	int ret;
 	unsigned i;
 
