@@ -18,7 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -248,13 +248,13 @@ int dane_state_init(dane_state_t * s, unsigned int flags)
 	ub_ctx_debugout(ctx, stderr);
 
 	if (!(flags & DANE_F_IGNORE_LOCAL_RESOLVER)) {
-		if ((ret = ub_ctx_resolvconf(ctx, NULL)) != 0) {
+		if (ub_ctx_resolvconf(ctx, NULL) != 0) {
 			gnutls_assert();
 			ret = DANE_E_INITIALIZATION_ERROR;
 			goto cleanup;
 		}
 
-		if ((ret = ub_ctx_hosts(ctx, NULL)) != 0) {
+		if (ub_ctx_hosts(ctx, NULL) != 0) {
 			gnutls_assert();
 			ret = DANE_E_INITIALIZATION_ERROR;
 			goto cleanup;
@@ -263,9 +263,8 @@ int dane_state_init(dane_state_t * s, unsigned int flags)
 
 	/* read public keys for DNSSEC verification */
 	if (!(flags & DANE_F_IGNORE_DNSSEC)) {
-		if ((ret =
-		     ub_ctx_add_ta_file(ctx,
-					(char *) UNBOUND_ROOT_KEY_FILE)) !=
+		if (ub_ctx_add_ta_file(ctx,
+				       (char *) UNBOUND_ROOT_KEY_FILE) !=
 		    0) {
 			gnutls_assert();
 			ret = DANE_E_INITIALIZATION_ERROR;
