@@ -660,6 +660,8 @@ read_crq_set(gnutls_x509_crq_t crq, const char *input_str, const char *oid)
 	free(lineptr);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 /* The input_str should contain %d or %u to print the default.
  */
 static int64_t read_int_with_default(const char *input_str, long def)
@@ -714,6 +716,7 @@ static int64_t read_int_with_default(const char *input_str, long def)
 
 	return l;
 }
+#pragma GCC diagnostic pop
 
 int64_t read_int(const char *input_str)
 {
@@ -848,6 +851,7 @@ const char *get_confirmed_pass(bool empty_ok)
 
 			pass = getpass("Enter password: ");
 			copy = strdup(pass);
+			CHECK_MALLOC(copy);
 			pass = getpass("Confirm password: ");
 		}
 		while (strcmp(pass, copy) != 0
@@ -1561,7 +1565,7 @@ int default_crl_number(unsigned char* serial, size_t *size)
  * that contains a default value that will be displayed to the user and
  * maximum size of the buffer that it can fill. When the function
  * returns, either the buffer is not modified to use the default value
- * or it's contents are changed to reflect the user-entered value.
+ * or its contents are changed to reflect the user-entered value.
  **/
 static
 void read_serial_value(unsigned char *serial, size_t *size, size_t max_size,
@@ -1880,10 +1884,11 @@ int get_tls_server_status(void)
 /* convert a printable IP to binary */
 static int string_to_ip(unsigned char *ip, const char *str)
 {
-	int len = strlen(str);
 	int ret;
 
 #if HAVE_IPV6
+	int len = strlen(str);
+
 	if (strchr(str, ':') != NULL || len > 16) {	/* IPv6 */
 		ret = inet_pton(AF_INET6, str, ip);
 		if (ret <= 0) {
