@@ -934,11 +934,7 @@ int listen_socket(const char *name, int listen_port, int socktype)
 	snprintf(portname, sizeof(portname), "%d", listen_port);
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_socktype = socktype;
-	hints.ai_flags = AI_PASSIVE
-#ifdef AI_ADDRCONFIG
-	    | AI_ADDRCONFIG
-#endif
-	    ;
+	hints.ai_flags = AI_PASSIVE;
 
 	if ((s = getaddrinfo(NULL, portname, &hints, &res)) != 0) {
 		fprintf(stderr, "getaddrinfo() failed: %s\n",
@@ -1135,7 +1131,7 @@ static void terminate(int sig)
 	*p++ = '\n';
 
 	write(STDERR_FILENO, buf, p - buf);
-	exit(1);
+	_exit(1);
 }
 
 
