@@ -113,6 +113,9 @@ void start(const char *prio, unsigned expect_max)
 
 	HANDSHAKE(client, server);
 
+	assert((gnutls_session_get_flags(server) & GNUTLS_SFLAGS_CLI_REQUESTED_OCSP) != 0);
+	assert((gnutls_session_get_flags(client) & GNUTLS_SFLAGS_CLI_REQUESTED_OCSP) != 0);
+
 	/* check gnutls_certificate_get_ours() - client side */
 	{
 		const gnutls_datum_t *mcert;
@@ -124,6 +127,9 @@ void start(const char *prio, unsigned expect_max)
 		}
 	}
 
+	assert(gnutls_certificate_type_get(server)==GNUTLS_CRT_X509);
+	assert(gnutls_certificate_type_get(client)==GNUTLS_CRT_X509);
+
 	/* check the number of certificates received and verify */
 	{
 		unsigned cert_list_size = 0;
@@ -258,9 +264,5 @@ void doit(void)
 {
 	start("NORMAL:-VERS-ALL:+VERS-TLS1.2", 0);
 	start("NORMAL:-VERS-ALL:+VERS-TLS1.3", 0);
-#ifndef ENABLE_TLS13
-	start("NORMAL", 0);
-#else
 	start("NORMAL", 1);
-#endif
 }
