@@ -84,7 +84,7 @@ static void try(void)
 				serverx509cred);
 
 	gnutls_priority_set_direct(server,
-				   "NORMAL",
+				   "NORMAL:-VERS-ALL:+VERS-TLS1.2",
 				   NULL);
 	gnutls_transport_set_push_function(server, server_push);
 	gnutls_transport_set_pull_function(server, server_pull);
@@ -109,7 +109,7 @@ static void try(void)
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_priority_set_direct(client, "NORMAL:-KX-ALL:+RSA", NULL);
+	ret = gnutls_priority_set_direct(client, "NORMAL:-VERS-ALL:+VERS-TLS1.2:-KX-ALL:+RSA", NULL);
 	if (ret < 0)
 		exit(1);
 
