@@ -23,11 +23,14 @@
 #ifndef GNUTLS_TESTS_UTILS_H
 #define GNUTLS_TESTS_UTILS_H
 
+#include <assert.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <string.h>
 #include <stdarg.h>
+#include <unistd.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/pkcs11.h>
 
@@ -158,7 +161,7 @@ inline static void _check_wait_status(int status, unsigned sigonly)
 		} else {
 			if (!sigonly) {
 				if (WEXITSTATUS(status) == 77)
-					exit(77);
+					_exit(77);
 				fail("Child died with status %d\n",
 				     WEXITSTATUS(status));
 			}
@@ -177,4 +180,32 @@ inline static void check_wait_status_for_sig(int status)
 	_check_wait_status(status, 1);
 }
 
+inline static unsigned int get_timeout(void) {
+	const char *envvar;
+	unsigned long int ul;
+
+	envvar = getenv("GNUTLS_TEST_TIMEOUT");
+	if (!envvar || *envvar == '\0')
+		return 20 * 1000;
+
+	ul = strtoul(envvar, NULL, 10);
+	assert(ul <= UINT_MAX);
+
+	return (unsigned int) ul;
+}
+
+inline static unsigned int get_dtls_retransmit_timeout(void) {
+	const char *envvar;
+	unsigned long int ul;
+
+	envvar = getenv("GNUTLS_TEST_DTLS_RETRANSMIT_TIMEOUT");
+	if (!envvar || *envvar == '\0')
+		return get_timeout() / 10;
+
+	ul = strtoul(envvar, NULL, 10);
+	assert(ul <= UINT_MAX);
+
+	return (unsigned int) ul;
+}
+
 #endif /* GNUTLS_TESTS_UTILS_H */
